unit AddLoginDlg;

Interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.StrUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.StdCtrls,

  StoreUser, StoreLine,
  VaxUser,
  VaxDefine, VaxServerSIP, VaxServerCOM,
  Codec;

type TAddLoginForm = class(TForm)

    Label1: TLabel;
    Label2: TLabel;
    Label4: TLabel;
    Label5: TLabel;

    Label7: TLabel;
    Label6: TLabel;
    Label8: TLabel;
    Label3: TLabel;

    Label14: TLabel;

    ComboLogin: TComboBox;
    ComboLoginRoute: TComboBox;

    btnAddLogin: TButton;
    btnRemoveLogin: TButton;
    btnClear: TButton;

    GroupBoxCodecs: TGroupBox;
    GroupBoxCallRouting: TGroupBox;
    GroupBoxType: TGroupBox;

    CheckBoxBlockAccount: TCheckBox;

    ListBoxEnableCodecs: TListBox;
    ListBoxDisableCodecs: TListBox;

    btnDisableCodec: TButton;
    btnEnableCodec: TButton;

    EditCallPrefix: TEdit;
    EditDialPrefix: TEdit;

    EditPassword: TEdit;

    RadioTypeAdmin: TRadioButton;
    RadioTypeAgent: TRadioButton;
    RadioTypeNormal: TRadioButton;

    procedure AddLogin_Click(Sender: TObject);
    procedure RemoveLogin_Click(Sender: TObject);
    procedure ClearLoginDlg_Click(Sender: TObject);

    procedure DisableCodec_Click(Sender: TObject);
    procedure EnableCodec_Click(Sender: TObject);

    procedure ComboBox_SelIndexChanged(Sender: TObject);

    public
    constructor Create(AOwner : TComponent); reintroduce; overload;

    private
    function AddLoginChecks(): Boolean;

    procedure LoadLoginAll();
    procedure LoadRouteLineAll();

    procedure ClearDlgUI();
    procedure ShowLoginDetails();

    procedure UpdateDlgUI(sLogin: String; sPassword: String; sAudioCodecList: String; bBlocked: Boolean; nVoiceTypeId: Integer; sUserPrefix, sDialPrefix: String; sToPeerName: String);

    procedure UpdateUserTypeUI(nUserType: Integer);
    procedure UpdateAudioCodecUI(sAudioCodecList: string);
    procedure UpdateRouteUI(sUserPrefix: string; sDialPrefix: string; sToPeerName: string);

    function GetUserTypeUI(): Integer;
    function GetAudioCodecUI(): string;
    procedure GetRouteUI(var sUserPrefix: String; var sDialPrefix: String; var sToPeerName: String);

    public
    procedure OnSetActive();

  end;

var
  AddLoginForm: TAddLoginForm;

implementation

{$R *.dfm}

  constructor TAddLoginForm.Create(AOwner : TComponent);
  begin
    inherited Create(AOwner);
    ClearDlgUI();
  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.AddLogin_Click(Sender: TObject);
  var
    sLoginName, sAudioCodecList: string;
    sUserPrefix, sDialPrefix, sToPeerName: string;
    nVoiceTypeId: Integer;
    bIsUpdated: Boolean;
  begin

    if(AddLoginChecks() = False) then
      Exit();

    bIsUpdated := False;
    sLoginName := ComboLogin.Text;

    if(ComboLogin.Items.IndexOf(sLoginName) <> -1) then
    begin
      if MessageDlg('Login already exists, do you want to continue?', mtConfirmation, [mbYes, mbNo], 0, mbNo) = mrNo then
      begin
        Exit();
      end;

      bIsUpdated := True;
    end;

    ////////////////////////////////////////////////////////////////////////////

    GetRouteUI(sUserPrefix, sDialPrefix, sToPeerName);

    sAudioCodecList := GetAudioCodecUI();
    nVoiceTypeId := GetUserTypeUI();

    ////////////////////////////////////////////////////////////////////////////

    if(TVaxUser.AddUser(sLoginName, EditPassword.Text, sAudioCodecList, CheckBoxBlockAccount.Checked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName) = False) then
    begin
      ShowMessage('Login could not be created.');
      Exit();
    end;

    if(ComboLogin.Items.IndexOf(sLoginName) = -1) Then
      ComboLogin.Items.Add(sLoginName);

    if bIsUpdated then
      ShowMessage('Login updated successfully.')
    else
      ShowMessage('Login added successfully.');

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.RemoveLogin_Click(Sender: TObject);
  var
  sLoginName: string;
  nIndex: Integer;
  begin

    sLoginName := ComboLogin.Text;

    if(Length(sLoginName) = 0) then
    begin
      ShowMessage('Please enter a valid login.');
      Exit();
    end;

    TVaxUser.RemoveUser(sLoginName);

    nIndex := ComboLogin.Items.IndexOf(sLoginName);

    if(nIndex <> -1) Then
      ComboLogin.Items.Delete(nIndex);

    ShowMessage('Login removed successfully.');
    ShowLoginDetails();

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.LoadLoginAll();
  var
    nIndex: Integer;
    objListUsers: TStringList;
  begin

    objListUsers := TStringList.Create();
    TStoreUser.GetUserAll(objListUsers);

    if(objListUsers.Count <= 0) then
        Exit();

    for nIndex := 0 to objListUsers.Count - 1 do
        ComboLogin.Items.Add(objListUsers[nIndex]);

    ComboLogin.ItemIndex := 0;
    ShowLoginDetails();

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.LoadRouteLineAll();
  var
    objListLines: TStringList;
    nIndex: Integer;
    sLineName: string;
  begin

    ComboLoginRoute.Clear;

    objListLines := TStringList.Create();
    TStoreLine.GetLineAll(objListLines);

    if(objListLines.Count <= 0) then
      Exit();

    for nIndex := 0 to objListLines.Count - 1 do
    begin
      sLineName := objListLines[nIndex];
      ComboLoginRoute.Items.AddObject(sLineName, TObject(TVaxServerCOM.VAX_PEER_TYPE_LINE));
    end;

    if(ComboLoginRoute.ItemIndex = -1) then
      ComboLoginRoute.ItemIndex := 0;

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.OnSetActive();
  begin
    LoadLoginAll();
    LoadRouteLineAll();
  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.ClearLoginDlg_Click(Sender: TObject);
  begin
    ClearDlgUI();
  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.ComboBox_SelIndexChanged(Sender: TObject);
  begin
    ShowLoginDetails();
  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.ShowLoginDetails();
  var
    sLoginName, sPassword, sUserPrefix: string ;
    sDialPrefix , sAudioCodecList, sToPeerName: string;

    nVoiceTypeId: Integer;
    bBlocked: Boolean;
  begin

    if(ComboLogin.Items.Count = 0) then
    begin
      ClearDlgUI();
      Exit();
    end;

    if(ComboLogin.ItemIndex = -1) then
      ComboLogin.ItemIndex := 0;

    sLoginName := ComboLogin.Text;

    TStoreUser.GetUser(sLoginName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
    UpdateDlgUI(sLoginName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.UpdateDlgUI(sLogin: String; sPassword: String; sAudioCodecList: String; bBlocked: Boolean; nVoiceTypeId: Integer; sUserPrefix, sDialPrefix: String; sToPeerName: String);
  begin

    EditPassword.Text := sPassword;

    CheckBoxBlockAccount.Checked := bBlocked;

    UpdateUserTypeUI(nVoiceTypeId);
    UpdateAudioCodecUI(sAudioCodecList);

    UpdateRouteUI(sDialPrefix, sUserPrefix, sToPeerName);

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.UpdateAudioCodecUI(sAudioCodecList: string);
  var
    nIndex: Integer;
    sCodecs, sCodecName: string;
  begin

    ListBoxEnableCodecs.Items.Clear();
    ListBoxDisableCodecs.Items.Clear();

    sCodecs := '';

    for nIndex := 0 to TVaxServerCOM.VAX_CODEC_TOTAL_COUNT - 1 do
        ListBoxDisableCodecs.Items.Add(TCodec.GetCodecName(nIndex));

    for nIndex := 1 to Length(sAudioCodecList) do
    begin
        sCodecName := TCodec.GetCodecName(sAudioCodecList[nIndex]);
        ListBoxEnableCodecs.Items.Add(sCodecName);
        ListBoxDisableCodecs.Items.Delete(ListBoxDisableCodecs.Items.IndexOf(sCodecName));
    end;

  end;

  function TAddLoginForm.GetAudioCodecUI(): string;
  var
    sCodecName, sCodecNo: string;
    sCodecList: string;
    nCount: Integer;
  begin

    sCodecList := ''; sCodecName := '';

    for nCount := 0 to ListBoxEnableCodecs.Items.Count - 1 do
    begin
      sCodecName := ListBoxEnableCodecs.Items[nCount];
      sCodecNo := TCodec.GetCodecNo(sCodecName);
      sCodecList := sCodecList + sCodecNo;
    end;

    Result := sCodecList;

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.UpdateRouteUI(sUserPrefix: string; sDialPrefix: string; sToPeerName: string);
  var
    nIndex: Integer;
  begin

    EditDialPrefix.Text := sDialPrefix;
    EditCallPrefix.Text := sUserPrefix;

    nIndex := ComboLoginRoute.Items.IndexOf(sToPeerName);

    if(nIndex <> -1) Then
    Begin
      ComboLoginRoute.ItemIndex :=  nIndex;
    End
    else
    Begin
      if(ComboLoginRoute.Items.Count > 0) then
        ComboLoginRoute.ItemIndex := 0;
    End;

  end;

  procedure TAddLoginForm.GetRouteUI(var sUserPrefix: String; var sDialPrefix: String; var sToPeerName: String);
  var
    nIndex: Integer;
  begin

    sUserPrefix := '';	sDialPrefix := '';
	  sToPeerName := '';

    nIndex := ComboLoginRoute.ItemIndex;

    if((nIndex = -1) OR (Length(EditCallPrefix.Text) = 0)) Then
      Exit();

    sUserPrefix := EditCallPrefix.Text;
	  sDialPrefix := EditDialPrefix.Text;

    sToPeerName := ComboLoginRoute.Text;

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.UpdateUserTypeUI(nUserType: Integer);
  begin

    if(nUserType = TVaxServerCOM.VAX_AUDIO_TYPE_ADMIN) then
      RadioTypeAdmin.Checked := True

    else if(nUserType = TVaxServerCOM.VAX_AUDIO_TYPE_USER) then
      RadioTypeAgent.Checked := True

    else if(nUserType = TVaxServerCOM.VAX_AUDIO_TYPE_NORMAL) then
      RadioTypeNormal.Checked := True
    else
      RadioTypeNormal.Checked := True;

  end;

  function TAddLoginForm.GetUserTypeUI(): Integer;
  begin

    if(RadioTypeAdmin.Checked = True) then
      Exit(TVaxServerCOM.VAX_AUDIO_TYPE_ADMIN);

    if(RadioTypeAgent.Checked = True) then
      Exit(TVaxServerCOM.VAX_AUDIO_TYPE_USER);

    if(RadioTypeNormal.Checked = True) then
      Exit(TVaxServerCOM.VAX_AUDIO_TYPE_NORMAL);

    Exit(TVaxServerCOM.VAX_AUDIO_TYPE_NORMAL);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.ClearDlgUI();
  begin

    ComboLogin.Text := '';
    EditPassword.Text := '';

    ListBoxEnableCodecs.Items.Clear();
    ListBoxDisableCodecs.Items.Clear();

    ListBoxEnableCodecs.Items.Add(TCodec.AUDIO_CODEC_NAME_G711U);
    ListBoxEnableCodecs.Items.Add(TCodec.AUDIO_CODEC_NAME_G711A);
    ListBoxEnableCodecs.Items.Add(TCodec.AUDIO_CODEC_NAME_ILBC);
    ListBoxEnableCodecs.Items.Add(TCodec.AUDIO_CODEC_NAME_GSM610);
    ListBoxEnableCodecs.Items.Add(TCodec.AUDIO_CODEC_NAME_G729);

    CheckBoxBlockAccount.Checked := false;
    RadioTypeNormal.Checked := True;

    EditCallPrefix.Text := '';
  	EditDialPrefix.Text := '';

    if(ComboLoginRoute.Items.Count > 0) then
        ComboLoginRoute.ItemIndex := 0;

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  function TAddLoginForm.AddLoginChecks(): Boolean;
  var
    sLoginName: string;
  begin

    if(Length(ComboLogin.Text) = 0) then
    begin
      ShowMessage('Please enter a valid login.');
      Exit(False);
    end;

    sLoginName := ComboLogin.Text;

    if(Not TVaxDefine.IsDigitAll(sLoginName)) Then
    begin
      ShowMessage('Login should contain only numerals 0 through 9. For example: 12, 324323, 800122, 2109756 etc.');
      Exit(False);
    end;

    if(Length(EditPassword.Text) = 0) then
    begin
      ShowMessage('Please enter login password.');
      Exit(False);
    end;

    if((Length(sLoginName) > 20) or (Length(EditPassword.Text ) > 20)) then
    begin
      ShowMessage('Login && password size should not be more than 20.');
      Exit(False);
    end;

    if(AnsiLeftStr(sLoginName, 2) = '00') then
    begin
      ShowMessage('Please do not enter 00 as login prefix. 00 prefix is use to prepare dial plan.');
      Exit(False);
    end;

    if(ListBoxEnableCodecs.Items.Count <= 0) then
    begin
      ShowMessage('Please select a codec.');
      Exit(False);
    end;

    Result := True;

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  procedure TAddLoginForm.DisableCodec_Click(Sender: TObject);
  begin

    if(ListBoxEnableCodecs.Items.Count <= 0) then
        Exit();

    if(ListBoxEnableCodecs.ItemIndex = -1) then
    begin
      ShowMessage('Please choose a codec first');
      Exit();
    end;

    ListBoxDisableCodecs.Items.Add(ListBoxEnableCodecs.Items[ListBoxEnableCodecs.ItemIndex]);
    ListBoxEnableCodecs.Items.Delete(ListBoxEnableCodecs.ItemIndex);

  end;

  procedure TAddLoginForm.EnableCodec_Click(Sender: TObject);
  begin

    if(ListBoxDisableCodecs.Items.Count <= 0) then
        Exit();

    if(ListBoxDisableCodecs.ItemIndex = -1) then
    begin
      ShowMessage('Please choose a codec first');
      Exit();
    end;

    ListBoxEnableCodecs.Items.Add(ListBoxDisableCodecs.Items[ListBoxDisableCodecs.ItemIndex]);
    ListBoxDisableCodecs.Items.Delete(ListBoxDisableCodecs.ItemIndex);

  end;

  //////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

end.


