unit RingGroupDlg;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs,

  StoreRingGroup, StoreUser,

  VaxRingGroup,
  VaxServerSIP, VaxServerCOM, VaxDefine, Vcl.ExtCtrls, Vcl.StdCtrls;

type

    TRingGroupForm = class(TForm)

    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;

    Label1: TLabel;
    Label2: TLabel;

    ComboRingGroupName: TComboBox;
    ComboProcessMode: TComboBox;

    Label3: TLabel;
    ListRingGroupAgent: TListBox;

    BtnAddAgent: TButton;
    BtnRemoveAgent: TButton;
    BtnClearAgent: TButton;

    BtnAddGroup: TButton;
    BtnRemoveGroup: TButton;
    BtnClearGroup: TButton;
    ComboAgentName: TComboBox;
    Label4: TLabel;
    ComboProcessType: TComboBox;

    procedure ComboRingGroupNameChange(Sender: TObject);

    procedure OnBtnRemoveGroupClick(Sender: TObject);
    procedure OnBtnClearGroupClick(Sender: TObject);
    procedure OnBtnAddAgentClick(Sender: TObject);
    procedure OnBtnRemoveAgentClick(Sender: TObject);

    procedure OnBtnAddGroupClick(Sender: TObject);
    procedure OnBtnClearAgentClick(Sender: TObject);

  private

    function AddRingGroupChecks(): boolean;
    function GetProcessMode(): Integer;
    function GetProcessType(): Integer;

    procedure AddUsers();
    procedure AddRingGroup();

    procedure AddProcessMode();
    procedure AddProcessType();

    procedure ClearDlgUI();
    procedure GetAgentNameAll(var objList: TStringList);

    procedure UpdateDlgUI();
    procedure ShowRingGroupDetails();

  public

    procedure OnSetActive();
    constructor Create(AOwner: TComponent); reintroduce; overload;

  end;

var
  RingGroupForm: TRingGroupForm;

implementation

{$R *.dfm}

  constructor TRingGroupForm.create(AOwner: TComponent);
  begin
    inherited Create(AOwner);
  end;

  procedure TRingGroupForm.ComboRingGroupNameChange(Sender: TObject);
  begin
    ShowRingGroupDetails();
  end;

  procedure TRingGroupForm.OnSetActive();
  begin
    UpdateDlgUI();
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TRingGroupForm.OnBtnAddGroupClick(Sender: TObject);
  var
    bIsUpdated: Boolean;
    objAgentList: TStringList;
  begin

	  if(AddRingGroupChecks() = False) Then
    begin
      Exit();
    end;

    bIsUpdated := False;

    if(ComboRingGroupName.Items.IndexOf(ComboRingGroupName.Text) <> -1) then
    begin
      if MessageDlg('RingGroup already exists, do you want to continue?', mtConfirmation, [mbYes, mbNo], 0, mbNo) = mrNo then
      begin
        Exit();
      end;

      bIsUpdated := True;
    end;

    objAgentList :=  TStringList.Create();
    GetAgentNameAll(objAgentList);

    if(TVaxRingGroup.AddRingGroup(ComboRingGroupName.Text, GetProcessMode(), GetProcessType(), objAgentList) <> True) then
    begin
      ShowMessage('Unable to add Ring Group.');
      Exit();
    end;

    if(ComboRingGroupName.Items.IndexOf(ComboRingGroupName.Text) = -1) Then
       ComboRingGroupName.Items.Add(ComboRingGroupName.Text);

    if bIsUpdated then
      ShowMessage('RingGroup updated successfully.')
    else
      ShowMessage('RingGroup added successfully.');

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TRingGroupForm.AddRingGroupChecks(): boolean;
  begin

    if(ComboRingGroupName.Text = '') then
    begin
      ShowMessage('Please enter RingGroup Name.');
      Exit(False);
    end;

    if(ComboProcessMode.Text = '') then
    begin
      ShowMessage('Please select Process Mode');
      Exit(False);
    end;

    if(ComboProcessType.Text = '') then
    begin
      ShowMessage('Please select Process Mode Type Free/Any');
      Exit(False);
    end;

    Result := True;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TRingGroupForm.OnBtnRemoveGroupClick(Sender: TObject);
  begin
    TVaxRingGroup.RemoveRingGroup(ComboRingGroupName.Text);
    ClearDlgUI();

    ComboRingGroupName.Items.delete(ComboRingGroupName.ItemIndex);
    ShowMessage('Ring Group Removed Successfully!');
  end;

  procedure TRingGroupForm.OnBtnClearGroupClick(Sender: TObject);
  begin
    ClearDlgUI();
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TRingGroupForm.GetProcessMode(): Integer;
  begin
     Result := ComboProcessMode.ItemIndex;
  end;

  function TRingGroupForm.GetProcessType(): Integer;
  begin
    Result := ComboProcessType.ItemIndex;
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TRingGroupForm.AddProcessMode();
  begin

    ComboProcessMode.Clear();

    ComboProcessMode.Items.Add(TVaxDefine.GROUP_ALGO_RING_ALL);
    ComboProcessMode.Items.Add(TVaxDefine.GROUP_ALGO_HUNT_RANDOM);
    ComboProcessMode.Items.Add(TVaxDefine.GROUP_ALGO_ROUND_ROBIN);
    ComboProcessMode.Items.Add(TVaxDefine.GROUP_ALGO_LEAST_ANSWERED);
    ComboProcessMode.Items.Add(TVaxDefine.GROUP_ALGO_LEAST_TALK_TIME);
    ComboProcessMode.Items.Add(TVaxDefine.GROUP_ALGO_LONGEST_WAITING);
    ComboProcessMode.Items.Add(TVaxDefine.GROUP_ALGO_PRIORITIZED_HUNT);

    ComboProcessMode.ItemIndex := 0;

  end;

  procedure TRingGroupForm.AddProcessType();
  begin

    ComboProcessType.Clear();

    ComboProcessType.Items.Add(TVaxDefine.GROUP_ALGO_MODE_TYPE_FREE);
    ComboProcessType.Items.Add(TVaxDefine.GROUP_ALGO_MODE_TYPE_ANY);

    ComboProcessType.ItemIndex := 0;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TRingGroupForm.OnBtnAddAgentClick(Sender: TObject);
  var
    sAgentName: string;
  begin
    sAgentName := ComboAgentName.Text;

    if(ListRingGroupAgent.Items.IndexOf(sAgentName) <> -1) Then
      Exit();

    ListRingGroupAgent.Items.Insert(0, sAgentName);
  end;

  procedure TRingGroupForm.OnBtnRemoveAgentClick(Sender: TObject);
  var
    nIndex: Integer;
    sAgentName: string;
  begin
    for nIndex:= ListRingGroupAgent.Items.count -1 downto 0 do
      if ListRingGroupAgent.selected[nIndex] then
      begin
        sAgentName := ListRingGroupAgent.Items[nIndex];
        ListRingGroupAgent.Items.delete(nIndex);
      end;
  end;

  procedure TRingGroupForm.OnBtnClearAgentClick(Sender: TObject);
  begin
    ListRingGroupAgent.Clear();
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TRingGroupForm.UpdateDlgUI();
  begin
    AddRingGroup();
    AddUsers();
    AddProcessMode();
    AddProcessType();
    ShowRingGroupDetails()
  end;

  procedure TRingGroupForm.ClearDlgUI();
  begin
    ComboRingGroupName.Text := '';

    if(ComboProcessMode.Items.Count > 0) Then
      ComboProcessMode.ItemIndex := 0;

    if(ComboProcessType.Items.Count > 0) Then
      ComboProcessType.ItemIndex := 0;

    if(ComboAgentName.Items.Count > 0) Then
      ComboAgentName.ItemIndex := 0;

    ListRingGroupAgent.Clear();
  end;

  procedure TRingGroupForm.GetAgentNameAll(var objList: TStringList);
  var
    sAgentName: String;
  begin

    for sAgentName in ListRingGroupAgent.Items do
    begin
      objList.Add(sAgentName);
    end;

  end;

  ////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////////////////////////////////////////////////////

  procedure TRingGroupForm.ShowRingGroupDetails();
  var
    objAgentNameList: TStringList;
    sAgentName: string;
    nProcessMode, nProcessType: Integer;
  begin

    if(ComboRingGroupName.Text = '') then
    begin
      Exit();
    end;

    objAgentNameList := TStringList.Create();

    if(TStoreRingGroup.GetGroup(ComboRingGroupName.Text, nProcessMode, nProcessType, objAgentNameList) = False) then
    begin
      Exit();
    end;

    if(ComboProcessMode.Items.Count > 0) Then
      ComboProcessMode.ItemIndex := nProcessMode;

    if(ComboProcessType.Items.Count > 0) Then
      ComboProcessType.ItemIndex := nProcessType;

    ListRingGroupAgent.Clear();

    for sAgentName in objAgentNameList do
    begin
      ListRingGroupAgent.Items.Add(sAgentName);
    end;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TRingGroupForm.AddUsers();
  var
    objLoginList: TStringList;
    sAgentName: string;
  begin

    ComboAgentName.Clear();

    objLoginList := TStringList.Create();
    TStoreUser.GetUserAll(objLoginList);

    for sAgentName in objLoginList do
    begin
      ComboAgentName.Items.Add(sAgentName);
    end;

    ComboAgentName.ItemIndex := 0;

  end;

  procedure TRingGroupForm.AddRingGroup();
  var
    objList: TStringList;
    sRingGroupName: string;
  begin

    objList := TStringList.Create();
    TStoreRingGroup.GetGroupNameAll(objList);

    ComboRingGroupName.Clear();

    for sRingGroupName in objList do
    begin
      ComboRingGroupName.Items.Add(sRingGroupName);
    end;

    ComboRingGroupName.ItemIndex := 0;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

end.
