unit VaxCallSessionRoute;

interface

uses
  System.SysUtils, System.Classes, System.StrUtils,
  VaxHashText;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

type TUserRouteDetails = class(TVaxHashTextData)

  public

  m_sUserPrefix, m_sToPeerName, m_sDialPrefix: String;

end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

type TLineRouteDetails = class(TVaxHashTextData)

  public

  m_sLineName, m_sToPeerName, m_sRoomName: String;
  m_sQueueName, m_sRingGroupName: String;
  m_bRejected: Boolean;

end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

type TVaxCallSessionRoute = class

  private

  m_objListUserRoute: TVaxHashText;
  m_objListLineRoute: TVaxHashText;

  private
  function FindUserRoute(sUserName: String): TUserRouteDetails;
  function FindLineRoute(sLineName: String): TLineRouteDetails;

  function GetUserRouteDetails(sUserName: String; var sUserPrefix, sDialPrefix: String; var sToPeerName: String): Boolean;
  function GetLineRouteDetails(sLineName: String; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String; var sQueueName: String; var sRingGroupName: String): Boolean;

  public
  constructor Create();
  destructor Destroy(); override;

  public
  procedure AddUserRoute(sUserName: String; sUserPrefix: String; sDialPrefix: String; sToPeerName: String);
  procedure RemoveUserRoute(sUserName: String);

  procedure AddLineRoute(sLineName: String; sToPeerName: String; bRejected: Boolean; sRoomName: String; sQueueName: String; sRingGroupName: String);
  procedure RemoveLineRoute(sLineName: String);

  function GetUserRouteInfo(sUserName: String; sDialNo: String; var sDialToNo: String; var sToPeerName: String): Boolean;
  procedure GetLineRouteInfo(sLineName: String; sDialNo: String; var sDialToNo: String; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String; var sQueueName: String; var sRingGroupName: String);

end;

implementation

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  Uses VaxServerSIP, VaxServerCOM;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  constructor TVaxCallSessionRoute.Create();
  begin
    m_objListUserRoute := TVaxHashText.Create(64);
    m_objListLineRoute := TVaxHashText.Create(64);
  end;

  destructor TVaxCallSessionRoute.Destroy();
  begin
    m_objListUserRoute.Destroy();
    m_objListLineRoute.Destroy();
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TVaxCallSessionRoute.FindUserRoute(sUserName: String): TUserRouteDetails;
  var
    objUserRoute: TUserRouteDetails;
  begin
    objUserRoute := TUserRouteDetails(m_objListUserRoute.FindData(sUserName));
    if(objUserRoute = nil) then Exit(nil);

    Result := objUserRoute;
  end;

  function TVaxCallSessionRoute.FindLineRoute(sLineName: String): TLineRouteDetails;
  var
    objLineRoute: TLineRouteDetails;
  begin
    objLineRoute := TLineRouteDetails(m_objListLineRoute.FindData(sLineName));
    if(objLineRoute = nil) then Exit(nil);

    Result := objLineRoute;
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TVaxCallSessionRoute.GetUserRouteDetails(sUserName: String; var sUserPrefix, sDialPrefix: String; var sToPeerName: String): Boolean;
  var
    objUserRoute: TUserRouteDetails;
  begin

    objUserRoute := FindUserRoute(sUserName);
    if(objUserRoute = nil) then Exit(False);

    sUserPrefix := objUserRoute.m_sUserPrefix;
    sDialPrefix := objUserRoute.m_sDialPrefix;

    sToPeerName := objUserRoute.m_sToPeerName;
    Result := True;

  end;

  function TVaxCallSessionRoute.GetLineRouteDetails(sLineName: String; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String; var sQueueName: String; var sRingGroupName: String): Boolean;
  var
    objLineRoute: TLineRouteDetails;
  begin

    objLineRoute := FindLineRoute(sLineName);
    if(objLineRoute = nil) then Exit(False);

    sToPeerName := objLineRoute.m_sToPeerName;

    bRejected := objLineRoute.m_bRejected;
    sRoomName := objLineRoute.m_sRoomName;

    sQueueName := objLineRoute.m_sQueueName;
    sRingGroupName := objLineRoute.m_sRingGroupName;

    Result := True;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxCallSessionRoute.AddUserRoute(sUserName: String; sUserPrefix: String; sDialPrefix: String; sToPeerName: String);
  var
    objUserRoute: TUserRouteDetails;
  begin

    RemoveUserRoute(sUserName);

    objUserRoute := TUserRouteDetails.Create();

    objUserRoute.m_sUserPrefix := sUserPrefix;
    objUserRoute.m_sDialPrefix := sDialPrefix;
    objUserRoute.m_sToPeerName := sToPeerName;

    m_objListUserRoute.AddData(sUserName, objUserRoute);

  end;

  procedure TVaxCallSessionRoute.RemoveUserRoute(sUserName: String);
  begin
    m_objListUserRoute.RemoveData(sUserName);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxCallSessionRoute.AddLineRoute(sLineName: String; sToPeerName: String; bRejected: Boolean; sRoomName: String; sQueueName: String; sRingGroupName: String);
  var
    objLineRoute: TLineRouteDetails;
  begin
    RemoveLineRoute(sLineName);

    objLineRoute := TLineRouteDetails.Create();

    objLineRoute.m_sToPeerName := sToPeerName;
    objLineRoute.m_bRejected := bRejected;
    objLineRoute.m_sRoomName := sRoomName;

    objLineRoute.m_sQueueName := sQueueName;
    objLineRoute.m_sRingGroupName := sRingGroupName;

    m_objListLineRoute.AddData(sLineName, objLineRoute);
  end;

  procedure TVaxCallSessionRoute.RemoveLineRoute(sLineName: String);
  begin
    m_objListLineRoute.RemoveData(sLineName);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TVaxCallSessionRoute.GetUserRouteInfo(sUserName: String; sDialNo: String; var sDialToNo: String; var sToPeerName: String): Boolean;
  var
    nUserPrefixLen: Integer;
    sUserPrefix, sDialPrefix: String;
    bResult: Boolean;
  begin

    bResult := GetUserRouteDetails(sUserName, sUserPrefix, sDialPrefix, sToPeerName);

    if ((bResult = False) OR (Length(sToPeerName) = 0) OR (Length(sUserPrefix) = 0)) then
    begin
      sDialToNo := sDialNo;
      sToPeerName := sDialNo;

      Exit(True);
    end;

    nUserPrefixLen := Length(sUserPrefix);

    // Check if first 'N' charachters are equal to sUserPrefix. If YES; Modify it according to dialplan.

    if(AnsiMidStr(sDialNo, 1, nUserPrefixLen) = sUserPrefix) then
    begin
      Delete(sDialNo, 1, nUserPrefixLen);
      sDialToNo := sDialPrefix + sDialNo;
    end
    else // else make it USER_TO_USER call
    begin
      sDialToNo := sDialNo;
      sToPeerName := sDialNo;
    end;

    Result := True;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxCallSessionRoute.GetLineRouteInfo(sLineName: String; sDialNo: String; var sDialToNo: String; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String; var sQueueName: String; var sRingGroupName: String);
  var
    bResult: Boolean;
  begin

    bResult := GetLineRouteDetails(sLineName, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);

    if(bResult = True) Then
    begin

      if(Length(sToPeerName) <> 0) Then
        sDialToNo := sToPeerName;

      Exit();

    end;

    sDialToNo := sDialNo;
    sToPeerName := sDialNo;

  end;

end.
