﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;

namespace ServerSIP
{
    public partial class RingGroupForm : Form
    {
        public RingGroupForm()
        {
            InitializeComponent();
        }

        private void BtnAddRingGroup_Click(object sender, EventArgs e)
        {
            if(ComboRingGroupName.Text == "" || ComboProcessMode.Text == "")
            {
                MessageBox.Show("Please fill all values.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            if(ComboRingGroupName.FindStringExact(ComboRingGroupName.Text) != -1)
            {
                if(MessageBox.Show("Ring Group already exists, do you want to continue?", "VaxVoIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                {
                    return;
                }
            }

            ArrayList objAgentList = new ArrayList();
            GetAgentNameAll(ref objAgentList);

            if (!cVaxRingGroup.AddRingGroup(ComboRingGroupName.Text, GetProcessMode(), GetProcessType(), objAgentList))
            {
                MessageBox.Show("Ring Group could not be added/updated or Please start the server first.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
                       
            if(!ComboRingGroupName.Items.Contains(ComboRingGroupName.Text))
                ComboRingGroupName.Items.Add(ComboRingGroupName.Text);
                        
            MessageBox.Show("Ring Group added/updated successfully.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void BtnRemoveRingGroup_Click(object sender, EventArgs e)
        {

            if(ComboRingGroupName.Text == "" || (!ComboRingGroupName.Items.Contains(ComboRingGroupName.Text)))
                return;
                        
            int nIndex = ComboRingGroupName.FindStringExact(ComboRingGroupName.Text);
            if(nIndex == -1) return;

            cVaxRingGroup.RemoveRingGroup(ComboRingGroupName.Text);
            ComboRingGroupName.Items.RemoveAt(nIndex);

            ClearDlgUI();

            if (ComboRingGroupName.Items.Count > 0)
                ComboRingGroupName.SelectedIndex = 0;

            ShowRingGroupDetails();

            MessageBox.Show("Ring Group removed successfully.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void BtnClearRingGroup_Click(object sender, EventArgs e)
        {
            ClearDlgUI();
        }

        private void BtnAddRingGroupAgent_Click(object sender, EventArgs e)
        {
            if(ListBoxLogin.FindStringExact(ComboAgentName.Text) != -1)
            {
                return;
            }

            ListBoxLogin.Items.Insert(0, ComboAgentName.Text);
        }

        private void BtnRemoveRingGroupAgent_Click(object sender, EventArgs e)
        {
            ListBoxLogin.Items.Remove(ListBoxLogin.SelectedItem);
        }

        private void BtnClearRingGroupAgent_Click(object sender, EventArgs e)
        {
            ClearDlgUI();
        }

        private void ClearDlgUI()
        {
            ComboRingGroupName.Text = "";

            if (ComboProcessMode.Items.Count > 0)
                ComboProcessMode.SelectedIndex = 0;

            if (ComboProcessType.Items.Count > 0)
                ComboProcessType.SelectedIndex = 0;

            if (ComboAgentName.Items.Count > 0)
                ComboAgentName.SelectedIndex = 0;

            ListBoxLogin.Items.Clear();
       }

        private void GetAgentNameAll(ref ArrayList objList)
        {
            foreach (String sAgentName in ListBoxLogin.Items)
            {
                objList.Add(sAgentName);
            }
        }

       private int GetProcessMode()
       {
           if(ComboProcessMode.SelectedIndex == -1)
           {
               MessageBox.Show("Please select a process mode.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
               return -1;
           }

           return ComboProcessMode.SelectedIndex;
       }

       private int GetProcessType()
       {
           if(ComboProcessType.SelectedIndex == -1)
           {
               MessageBox.Show("Please select a process type.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
               return -1;
           }

           return ComboProcessType.SelectedIndex;
       }

       private void AddProcessMode()
       {
           ComboProcessMode.Items.Clear();

           ComboProcessMode.Items.Add(cVaxDefine.GROUP_PROCESS_MODE_RING_ALL);
           ComboProcessMode.Items.Add(cVaxDefine.GROUP_PROCESS_MODE_HUNT_RANDOM);
           ComboProcessMode.Items.Add(cVaxDefine.GROUP_PROCESS_MODE_ROUND_ROBIN);
           ComboProcessMode.Items.Add(cVaxDefine.GROUP_PROCESS_MODE_LEAST_ANSWERED);
           ComboProcessMode.Items.Add(cVaxDefine.GROUP_PROCESS_MODE_LEAST_TALK_TIME);
           ComboProcessMode.Items.Add(cVaxDefine.GROUP_PROCESS_MODE_LONGEST_WAITING);
           ComboProcessMode.Items.Add(cVaxDefine.GROUP_PROCESS_MODE_PRIORITIZED_HUNT);

           ComboProcessMode.SelectedIndex = 0;
       }

       private void AddProcessType()
       {
           ComboProcessType.Items.Clear();

           ComboProcessType.Items.Add(cVaxDefine.GROUP_PROCESS_MODE_PROCESS_TYPE_FREE);
           ComboProcessType.Items.Add(cVaxDefine.GROUP_PROCESS_MODE_PROCESS_TYPE_ANY);
           
           ComboProcessMode.SelectedIndex = 0;
       }

       private void AddRingGroups()
       {
           ComboRingGroupName.Items.Clear();

           ArrayList objList = new ArrayList();
           cStoreRingGroup.GetGroupNameAll(ref objList);

           if (objList.Count <= 0)
               return;

           foreach (String sGroupName in objList)
           {
               ComboRingGroupName.Items.Add(sGroupName);
           }

           if(ComboRingGroupName.Items.Count > 0)
               ComboRingGroupName.SelectedIndex = 0;
       }

       private void AddUsers()
       {
           ComboAgentName.Items.Clear();

           ArrayList objList = new ArrayList();
           cStoreUser.GetUserAll(ref objList);

           if (objList.Count <= 0) return;

           for (int nIndex = 0; nIndex < objList.Count; nIndex++)
               ComboAgentName.Items.Add(objList[nIndex]);

           if(ComboAgentName.SelectedIndex == -1 && ComboAgentName.Items.Count > 0)
               ComboAgentName.SelectedIndex = 0;

       }
       public void OnSetActive()
       {
           AddUsers();

           AddProcessMode();
           AddProcessType();
           
           AddRingGroups();
           ShowRingGroupDetails();
       }

       private void ComboRingGroupName_SelectedIndexChanged(object sender, EventArgs e)
       {
           ShowRingGroupDetails();
       }

       private void ShowRingGroupDetails()
       {
           String sRingGroupName = ComboRingGroupName.Text;
           if(sRingGroupName.Length == 0) return;
                      
           int nProcessMode = -1, nProcessType = -1;

           ArrayList objAgentList = new ArrayList();

           if (!cStoreRingGroup.GetGroup(sRingGroupName, ref nProcessMode, ref nProcessType, ref objAgentList))
               return;

           if (ComboProcessMode.Items.Count > 0)
               ComboProcessMode.SelectedIndex = 0;

           if (ComboProcessType.Items.Count > 0)
               ComboProcessType.SelectedIndex = 0;

           ListBoxLogin.Items.Clear();

           foreach (String sAgentName in objAgentList)
               ListBoxLogin.Items.Add(sAgentName);
       }
    }
}
