﻿using System;
using System.Collections;

namespace ServerSIP
{
    class cVaxQueue
    {
        static cVaxQueueList m_objQueueList = null;

        public cVaxQueue()
        {
            if (cVaxQueue.m_objQueueList == null)
                cVaxQueue.m_objQueueList = new cVaxQueueList();
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////

        public static void ResetAll()
        {
            cVaxQueue.m_objQueueList = new cVaxQueueList();
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////

        private static void AddQueueList(String sQueueName, String sMusicFileName, String sDialToneFileName)
        {
            cVaxQueue.RemoveQueueList(sQueueName);

            int nMusicWaveId = cVaxServerSIP.mVaxSIP.LoadWaveFile(sMusicFileName);
            int nDialToneWaveId = cVaxServerSIP.mVaxSIP.LoadWaveFile(sDialToneFileName);

            cVaxQueue.m_objQueueList.AddQueue(sQueueName, nMusicWaveId, nDialToneWaveId);
        }

        private static void RemoveQueueList(String sQueueName)
        {
            int nMusicWaveId = cVaxQueue.m_objQueueList.GetQueueMusicId(sQueueName);
            int nDialToneWaveId = cVaxQueue.m_objQueueList.GetQueueDialToneId(sQueueName);

            cVaxServerSIP.mVaxSIP.UnLoadWaveID(nMusicWaveId);
            cVaxServerSIP.mVaxSIP.UnLoadWaveID(nDialToneWaveId);

            cVaxQueue.m_objQueueList.RemoveQueue(sQueueName);
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////

        public void OnIncomingCall(int nSessionId, String sCallerName, String sCallerId, String sQueueName, ref Boolean bProcessed)
        {
            bProcessed = false;

            if (sQueueName.Length == 0)
                return;

            if (!cVaxServerSIP.mVaxSIP.AddCallSessionToQueue(sQueueName, sCallerName, sCallerId, nSessionId, 20))
            {
                return;
            }

            bProcessed = true;
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////

        public void OnQueuePlayWaveStarted(int nEventType, String sQueueName, int nSessionId)
        {
            if (nEventType == cVaxServerCOM.VAX_QUEUE_PLAY_WAVE_EVENT_TYPE_MUSIC)
            {
                int nQueueMusicId = cVaxQueue.m_objQueueList.GetQueueMusicId(sQueueName);
                if (nQueueMusicId == -1) return;

                cVaxServerSIP.mVaxSIP.PlayWaveStartToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, nQueueMusicId, true, 0);
                return;
            }

            int nQueueDialToneId = cVaxQueue.m_objQueueList.GetQueueDialToneId(sQueueName);
            if (nQueueDialToneId == -1) return;

            cVaxServerSIP.mVaxSIP.PlayWaveStartToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, nQueueDialToneId, true, 0);
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////

        public void OnQueuePlayWaveEnded(int nEventType, String sQueueName, int nSessionId)
        {
            cVaxServerSIP.mVaxSIP.PlayWaveStopToCallSession(nSessionId, -1, -1);
        }

        public static Boolean AddQueue(String sQueueName, int nProcessMode, int nProcessType, ArrayList objAgentList, String sMusicFileName, String sDialToneFileName)
        {
            if (cVaxServerSIP.mVaxSIP.IsInitialized() == false || (sQueueName.Length == 0))
                return false;

            RemoveQueue(sQueueName);

            if (!cVaxServerSIP.mVaxSIP.AddQueue(sQueueName))
                return false;

            foreach (String sAgentName in objAgentList)
            {
                cVaxServerSIP.mVaxSIP.AddQueueAgent(sQueueName, sAgentName);
            }

            cVaxServerSIP.mVaxSIP.SetQueueProcessMode(sQueueName, nProcessMode, nProcessType);
            AddQueueList(sQueueName, sMusicFileName, sDialToneFileName);
                        
            cStoreQueue.AddQueue(sQueueName, nProcessMode, nProcessType, objAgentList, sMusicFileName, sDialToneFileName);
            return true;
        }

        public static void RemoveQueue(String sQueueName)
        {
            if (cVaxServerSIP.mVaxSIP.IsInitialized() == false || (sQueueName.Length == 0))
                return;

            RemoveQueueList(sQueueName);
            cVaxServerSIP.mVaxSIP.RemoveQueue(sQueueName);
            cStoreQueue.RemoveQueue(sQueueName);
        }

        public static void LoadQueue()
        {
            if (!cVaxServerSIP.mVaxSIP.IsInitialized())
                return;

            ArrayList objListQueue = new ArrayList();
            cStoreQueue.GetQueueNameAll(ref objListQueue);

            if (objListQueue.Count == 0)
                return;

            ArrayList objAgentList = new ArrayList();
            String sMusicFileName = "", sDialToneFileName = "";
            int nProcessMode = -1, nProcessType = -1;

            foreach (String sQueueName in objListQueue)
            {
                cStoreQueue.GetQueue(sQueueName, ref nProcessMode, ref nProcessType, ref objAgentList, ref sMusicFileName, ref sDialToneFileName);
                AddQueue(sQueueName, nProcessMode, nProcessType, objAgentList, sMusicFileName, sDialToneFileName);
            }
        }
    }
}
