﻿using System;
using System.Collections;

namespace ServerSIP
{
    public class cVaxServerSIP : cVaxServerCOM
    {
        public StatusLogForm m_objStatusLogDlg = null;
        public static cVaxServerSIP mVaxSIP = null;

        int m_nDialToneId = -1;
        
        Boolean m_bInitialized = false;
        cVaxCallSessionList m_objSessionList = null;

        #region Methods

        public cVaxServerSIP()
        {
            mVaxSIP = this;
            ResetAll();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////

        private void ResetAll()
        {
            m_objSessionList = new cVaxCallSessionList();

            cVaxUser.ResetAll();
            cVaxCallSession.ResetAll();
            cVaxQueue.ResetAll();

            cVaxStealthListen.ResetAll();
            cVaxCallParking.ResetAll();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////

        public Boolean StartServer(int nListenPort)
        {
            StopServer();

            base.SetLicenseKey("TRIAL-LICENSE-KEY");

            if (!base.Initialize(""))
                return false;

            if (!base.OpenNetworkUDP("", nListenPort))
            {
                UnInitialize();
                return false;
            }

            if (!base.OpenNetworkTCP("", nListenPort))
            {
                UnInitialize();
                return false;
            }

            if (!base.OpenNetworkTLS("", nListenPort + 1, ""))
            {
                UnInitialize();
                return false;
            }

            // base.AttackDetectScanSIP("demo.vaxvoip.com");
            // base.AttackDetectFloodSIP(1000);
            // base.AttackDetectBruteForceSIP(10, 2);

            base.SetListenPortRangeRTP(20000, 30000);
            base.AudioSessionLost(true, 30); // Enable voice session lost for 30 seconds.
            base.VideoCOMM(true);

            base.AddLine("AnonymousUDP", cVaxServerCOM.VAX_LINE_TYPE_UDP, "", "", "", "", "", "255.255.255.255", -1, "");
            base.AddLine("AnonymousTCP", cVaxServerCOM.VAX_LINE_TYPE_TCP, "", "", "", "", "", "255.255.255.255", -1, "");
            base.AddLine("AnonymousTLS", cVaxServerCOM.VAX_LINE_TYPE_TLS, "", "", "", "", "", "255.255.255.255", -1, "");

            base.AddCallPickUpGroupRouteUID("#5#"); // OnIncomingCall(), OnCallSessionTransferBlind(), sRouteUID Parameter.
            base.AddStealthListenRouteUID("#1#");   // OnIncomingCall(), OnCallSessionTransferBlind(), sRouteUID Parameter.
            base.AddCallParkRouteUID("#3#");        // OnIncomingCall(), OnCallSessionTransferBlind(), sRouteUID Parameter.

            //base.ActivatePushSIP(true, 1);

            m_bInitialized = true;
            OnInitialize();

            return true;
        }

        public void StopServer()
        {
            m_bInitialized = false;
            m_nDialToneId = -1;

            base.UnInitialize();
            ResetAll();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        public Boolean IsInitialized()
        {
            return m_bInitialized;
        }

        public Boolean AcceptCallSession(int nSessionId, String sCallerName, String sCallerId, String sDialNo, String sToPeerName, String sRouteUID)
        {
            return base.AcceptCallSession(nSessionId, sCallerName, sCallerId, sDialNo, sToPeerName, sRouteUID, 30);
        }
                
        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public Boolean DialToneToCallSession(Boolean bEnable, int nSessionId)
        {
            if (m_nDialToneId == -1)
                return true;

            return base.DialToneToCallSession(bEnable, nSessionId, m_nDialToneId);
        }
                        
        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private Boolean LoadMusicDialTone(String sFileName)
        {
            if (m_nDialToneId != -1)
               base.UnLoadWaveID(m_nDialToneId);
                
            m_nDialToneId = base.LoadWaveFile(sFileName);
            return (m_nDialToneId != -1);
        }
                        
        public void LoadMusicFiles(String sMusicHold, String sMusicDialTone, String sMusicCallPark)
        {
            base.LoadMusicHold(sMusicHold);
            LoadMusicDialTone(sMusicDialTone);

            cVaxCallParking.LoadMusic(sMusicCallPark);
            cVaxStealthListen.LoadMusic(sMusicCallPark);

            cStoreCOMM.SetMusicFiles(sMusicHold, sMusicDialTone, sMusicCallPark);
        }

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////

        #endregion

        #region Events

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionCreated(int nSessionId, int nReasonCode)
        {
            m_objSessionList.AddSession(nSessionId);
        }

        protected override void OnCallSessionClosed(int nSessionId, int nChannelId, int nReasonCode)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);

            if (objCallSession != null)
                objCallSession.OnCallSessionClosed(nSessionId, nChannelId, nReasonCode);

            m_objSessionList.RemoveSession(nSessionId);
        }

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        protected override void OnCallSessionAccessAudioPCM(int nSessionId, int nChannelId, int nAccessType, Byte[] aDataPCM, int nSizePCM, int nTypePCM)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionAccessAudioPCM(nSessionId, nAccessType, aDataPCM, nSizePCM);
        }

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionPushSIP(int nSessionId, int nChannelId, String sCallerName, String sCallerId, int nProviderPN, String sProjectPN, String sDevicePN)
        {
            // Use OnCallSessionPushSIP() event to send Push-Notification to Android and iOS based devices. 
            // OnCallSessionPushSIP() event triggers only if softphone or SIP client supports Push-SIP RFC.
            // VaxVoIP Phone SDK iOS & Android already support Push-SIP RFC.

            //base.ActivatePushSIP() VaxServerSDK Exported method. Please search in this Class. Please use it in StartServer()  function.

            /*
             
            if(nProviderPN == cVaxServerCOM.VAX_PUSH_PROVIDER_IOS)
            {
                Send Notification by using HTTP, Curl.exe or any other method to iOS device. sDevicePN is device token. sProjectPN is APP Name.
            }
            else if (nProviderPN == cVaxServerCOM.VAX_PUSH_PROVIDER_ANDROID)
            {
                Send Notification by using HTTP, Curl.exe or any other method to Android device. sDevicePN is device token. sProjectPN is APP Name.
            }
              
            */
        }

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnIncomingCall(int nSessionId, String sCallerName, String sCallerId, String sDialNo, int nFromPeerType, String sFromPeerName, String sRouteUID, int nRouteType, String sRouteName, String sUserAgentName, String sFromIP, int nFromPort)
        {
            if (sRouteUID.Length != 0)
            {
                if (nRouteType == VAX_ROUTE_TYPE_CALL_PARK)
                {
                    cVaxCallParking.OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, sRouteUID);
                    return;
                }

                if (nRouteType == VAX_ROUTE_TYPE_STEALTH_LISTEN)
                {
                    cVaxStealthListen.OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, nFromPeerType, sFromPeerName, sRouteUID);
                    return;
                }

                AcceptCallSession(nSessionId, sCallerName, sCallerId, sDialNo, "", sRouteUID);
                return;
            }

            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;
                        
            DialToneToCallSession(true, nSessionId);
            objCallSession.OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, nFromPeerType, sFromPeerName, sUserAgentName, sFromIP, nFromPort);
        }
                
        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionTransferBlind(int nTransfererSessionId, int nTransfererChannelId, String sTransferer, String sTransferee, String sTransferTo, String sRouteUID, int nRouteType, String sRouteName)
        {
            if (sRouteUID.Length != 0)
            {
                if (nRouteType == VAX_ROUTE_TYPE_CALL_PARK)
                {
                    cVaxCallParking.OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferee, sTransferTo, sRouteUID);
                    return;
                }

                base.AcceptTransferBlind(nTransfererSessionId, sTransferee, sTransferee, sTransferTo, "", sRouteUID, 30);
                return;
            }

            cVaxCallSession objCallSession = m_objSessionList.GetSession(nTransfererSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferer, sTransferee, sTransferTo);
        }

        protected override void OnCallSessionTransferConsult(int nTransfererSessionId, int nTransfererChannelId, int nTransferToSessionId, int nTransferToChannelId, String sTransferer, String sTransferee, String sTransferTo)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nTransfererSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionTransferConsult(nTransfererSessionId, nTransfererChannelId, nTransferToSessionId, nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnDetectedDigitDTMF(int nSessionId, int nChannelId, String sDigitDTMF, int nTypeDTMF)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnDetectedDigitDTMF(nSessionId, nChannelId, sDigitDTMF);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        protected override void OnLineRegisterTrying(String sLineName)
        {
            cVaxLine.OnLineRegisterTrying(sLineName);
        }

        protected override void OnLineRegisterFailed(String sLineName, int nStatusCode, String sReasonPhrase)
        {
            cVaxLine.OnLineRegisterFailed(sLineName, nStatusCode, sReasonPhrase);
        }

        protected override void OnLineRegisterSuccess(String sLineName)
        {
            cVaxLine.OnLineRegisterSuccess(sLineName);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        protected override void OnRegisterUser(String sUserLogin, String sDomain, String sUserAgentName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser.OnRegisterUser(sUserLogin, sDomain, sUserAgentName, sFromIP, nFromPort, nRegId);
        }

        protected override void OnRegisterUserSuccess(String sUserLogin, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser.OnRegisterUserSuccess(sUserLogin, sFromIP, nFromPort, nRegId);
        }

        protected override void OnRegisterUserFailed(String sUserLogin, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser.OnRegisterUserFailed(sUserLogin, sFromIP, nFromPort, nRegId);
        }
                
        protected override void OnUnRegisterUser(String sUserLogin)
        {    
            cVaxUser.OnUnRegisterUser(sUserLogin);
        }

        ///////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////

        protected override void OnChatMessageText(int nChatMsgId, String sMsgFrom, String sMsgTo, String sMsgText, int nFromPeerType, String sFromPeerName, String sFromIP, int nFromPort)
        {
            base.AcceptChatMessage(nChatMsgId, sMsgTo);
        }
        protected override void OnChatMessageTyping(int nChatMsgId, String sMsgFrom, String sMsgTo, Boolean bIsTypingStart, int nFromPeerType, String sFromPeerName, String sFromIP, int nFromPort)
        {
            base.AcceptChatMessage(nChatMsgId, sMsgTo);
        }
        protected override void OnChatStatusSubscribe(int nSubscrbId, String sMsgFrom, String sMsgTo, int nFromPeerType, String sFromPeerName, String sFromIP, int nFromPort)
        {
            base.AcceptChatStatusSubscribe(nSubscrbId, sMsgFrom, sMsgTo, sMsgTo);
        }

        ///////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////

        protected override void OnBusyLampSubscribe(int nSubScrbId, String sUserName, String sToUserName, String sFromIP, int nFromPort)
        {
            base.BusyLampSubscribeAccept(nSubScrbId, true, 30);
        }

        protected override void OnBusyLampSendStatus(String sFromUserName, String sToUserName, int nStateId)
        {
            base.BusyLampSendStatus(sFromUserName, sToUserName, nStateId);
        }

        ///////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////

        protected override void OnQueuePlayWaveStarted(int nEventType, String sQueueName, int nSessionId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnQueuePlayWaveStarted(nEventType, sQueueName, nSessionId);
        }

        protected override void OnQueuePlayWaveEnded(int nEventType, String sQueueName, int nSessionId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnQueuePlayWaveEnded(nEventType, sQueueName, nSessionId);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnStealthListenerAttached(int nAttachType, int nStealthSessionId, int nAttachSessionId, int nAttachChannelId)
        {
            cVaxStealthListen.OnStealthListenerAttached(nAttachType, nStealthSessionId, nAttachSessionId, nAttachChannelId);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
        protected override void OnCallSessionConnected(int nSessionId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionConnected(nSessionId);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
        protected override void OnCallSessionMediaDetectStartedVAD(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectStartedVAD(nSessionId, nChannelId);
        }

        protected override void OnCallSessionMediaDetectCompletedVAD(int nSessionId, int nChannelId, Byte[] aDataPCM, int nSizePCM, int nTypePCM)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectCompletedVAD(nSessionId, nChannelId, aDataPCM, nSizePCM, nTypePCM);
        }

        protected override void OnCallSessionMediaDetectExpiredVAD(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectExpiredVAD(nSessionId, nChannelId);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public void GenStatusLog(String sStatus)
        {
            OnStatusLog(sStatus);
        }

        protected override void OnVaxErrorLog(String sFuncName, int nErrorCode, String sErrorMsg)
        {
            String sMsg = "[Method: " + sFuncName + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
            OnStatusLog(sMsg);
        }

        protected override void OnCallSessionErrorLog(int nSessionId, int nChannelId, int nErrorCode, String sErrorMsg)
        {
            String sMsg = "[Session: " + nSessionId.ToString() + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
            OnStatusLog(sMsg);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
        
        private void OnInitialize()
        {
            if (!cVaxSettingDefault.LoadUserDefault())
            {
                cVaxUser.LoadUser();
            }

            if (!cVaxSettingDefault.LoadQueueDefault())
            {
                cVaxQueue.LoadQueue();
            }

            if (!cVaxSettingDefault.LoadRingGroupDefault())
            {
                cVaxRingGroup.LoadRingGroup();
            }

            if (!cVaxSettingDefault.LoadCallPickUpGroupDefault())
            {
                cVaxPickUpGroup.LoadCallPickUpGroup();
            }

            if (!cVaxSettingDefault.LoadRoomDefault())
            {
                cVaxRoom.LoadRoom();
            }

            if (!cVaxSettingDefault.LoadLineDefault())
            {
                cVaxLine.LoadLine();
            }
        }

        private void OnStatusLog(String sMsg)
        {
            m_objStatusLogDlg.OnStatusLog(sMsg);
        }
                
        #endregion
    }
}
