﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace AutoDialer
{

    public partial class MainForm : Form
    {
        cVaxServerExCOM m_objServerExCOM;

        UInt32 m_nPhoneNoId;
        int m_nConcurrentCallCount;
                                        
        public MainForm()
        {
            m_objServerExCOM = new cVaxServerExCOM(this);

            DateTime epoch = new DateTime(1970, 1, 1);
            DateTime obj = epoch.AddSeconds(8);
                        
            String sTime = String.Format("{0}:{1}:{2}", obj.Hour, obj.Minute, obj.Second);

            m_nPhoneNoId = 0;
            m_nConcurrentCallCount = 0;

            InitializeComponent();

            int nLineType = cVaxServerCOM.VAX_LINE_TYPE_UDP;

            string sDisplayName = "", sUserName = "";
            string sAuthUser = "", sAuthPwd = "";
            String sDomainRealm = "", sServerAddr = "";
            
            int nServerPort = 5060;

            cStoreLine.GetLine(cVaxServerExCOM.VAX_ACCOUNT_LINE_SIP, ref nLineType, ref sDisplayName, ref sUserName, ref sAuthUser, ref sAuthPwd, ref sDomainRealm, ref sServerAddr, ref nServerPort);

            EditDisplayName.Text = sDisplayName;
            EditUserName.Text = sUserName;
            
            EditAuthName.Text = sAuthUser;
            EditPassword.Text = sAuthPwd;
            
            EditDomain.Text = sDomainRealm;
            EditServerAddr.Text = sServerAddr;
            EditServerPort.Text = Convert.ToString(nServerPort);

            int nListenPortSIP = cStoreCOMM.GetListenPortSIP();
            EditListenPort.Text = nListenPortSIP.ToString();
        }

        private void btnStartServer_Click(object sender, EventArgs e)
        {
            m_objServerExCOM.SetLicenseKey("TRIAL-LICENSE-KEY");

            Boolean bStatusRun = m_objServerExCOM.Initialize(Convert.ToInt32(EditListenPort.Text));

            if (bStatusRun == false)
            {
                LabelServerStatus.Text = "UNABLE TO START SIP SERVER (SEE ERROR LOG).";
                return;
            }

            if (check_DiagnosticLog.Checked)
                m_objServerExCOM.DiagnosticLogSIP(true, true);

            cStoreCOMM.SetListenPortSIP(Convert.ToInt32(EditListenPort.Text));
            TimerCallCount.Enabled = true;
            
            LabelServerStatus.Text = "SIP SERVER AUTO DIALER IS RUNNING.";
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            TimerCallCount.Enabled = false;
            m_objServerExCOM.UnInitialize();
            
            LabelServerStatus.Text = "SIP SERVER AUTO DIALER IS NOT RUNNING.";
            LabelTotalCalls.Text = "0";

            btnRegisterAccountSIP.Text = "Online";
        }

        private void btnRegisterAccountSIP_Click(object sender, EventArgs e)
        {
            if (!m_objServerExCOM.IsInitialized())
            {
                MessageBox.Show("Please click the start button first.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            
            if(m_objServerExCOM.IsAddedAccountSIP())
            {
                m_objServerExCOM.UnRegisterAccountSIP();
                btnRegisterAccountSIP.Text = "Online";
                return;
            }

            if (!m_objServerExCOM.RegisterAccountSIP(EditDisplayName.Text, EditUserName.Text, EditAuthName.Text, EditPassword.Text, EditDomain.Text, EditServerAddr.Text, (Convert.ToInt32(EditServerPort.Text))))
                return;

            cStoreLine.AddLine(cVaxServerExCOM.VAX_ACCOUNT_LINE_SIP, cVaxServerCOM.VAX_LINE_TYPE_UDP, EditDisplayName.Text, EditUserName.Text, EditAuthName.Text, EditPassword.Text, EditDomain.Text, EditServerAddr.Text, Convert.ToInt32(EditServerPort.Text));
            btnRegisterAccountSIP.Text = "Offline";
        }  
           
        private void TimerGeneral_Tick(object sender, EventArgs e)
        {
            LabelTotalCalls.Text = m_objServerExCOM.GetCallCount().ToString();
        }

        public void OnVaxStatusLog(string sMsg)
        {
            ListStatus.Items.Add(sMsg);
            ListStatus.TopIndex = ListStatus.Items.Count - 1;
        }
        
        public void OnPhoneNoStatusLog(UInt32 nPhoneNoId, string sPhoneNo, string sStatus, string sDigitDTMF, string sDuration)
        {
            UpdateListItem(nPhoneNoId, sPhoneNo, sStatus, sDigitDTMF, sDuration);
        }

        private void btnClearLog_Click(object sender, EventArgs e)
        {
            ListStatus.Items.Clear();
        }

        private void btn_StartAutoDialer_Click(object sender, EventArgs e)
        {
            if (!m_objServerExCOM.IsAddedAccountSIP())
            {
                MessageBox.Show("SIP account is not added or registered.", "VaxVoIP SIP SDK", MessageBoxButtons.OK);
                return;
            }

            if (btn_StartAutoDialer.Text == "Stop Dialing")
            {
                TimerAutoDialer.Enabled = false;
                btn_StartAutoDialer.Text = "Start Dialing";
                return;
            }

            m_nConcurrentCallCount = Convert.ToInt32(EditConcurrentCalls.Text);

            TimerAutoDialer.Enabled = true;

            m_objServerExCOM.LoadMusicAutoDialer(EditWaveFileDialer.Text);

            m_objServerExCOM.EnableDetectDTMF(check_DetectDTMF.Checked);
            m_objServerExCOM.EnableDetectAMD(check_DetectAMD.Checked);

            m_objServerExCOM.SetPlayWaveConnected(check_PlayWave.Checked);
            btn_StartAutoDialer.Text = "Stop Dialing";
        }

        private void InsertListItem(string sPhoneNo, string sPhoneStatus, string sDigitDTMF, string sDuration)
        {
            sPhoneNo.Trim();
            sPhoneStatus.Trim();

            listView_PhoneNo.Items.Add(new cListViewItemEx(m_nPhoneNoId, sPhoneNo, sPhoneStatus, sDigitDTMF, sDuration));

            m_nPhoneNoId++;
            
            listView_PhoneNo.TopItem = listView_PhoneNo.Items[listView_PhoneNo.Items.Count - 1];
        }

        private void UpdateListItem(UInt32 nPhoneNoId, string sPhoneNo, string sPhoneStatus, string sDigitDTMF, string sDuration)
        {
            foreach (ListViewItem LVitem in listView_PhoneNo.Items)
            {
                if (((cListViewItemEx)LVitem).m_nPhoneNoId == nPhoneNoId) 
                {
                    if(sPhoneStatus.Length != 0)
                        LVitem.SubItems[1].Text = sPhoneStatus;

                    if (sDigitDTMF.Length != 0)
                        LVitem.SubItems[2].Text = sDigitDTMF;
                    
                    if(sDuration.Length != 0)
                        LVitem.SubItems[3].Text = sDuration;

                    break;
                }
            }
        }

        private void btnAddPhoneNo_Click(object sender, EventArgs e)
        {
            InsertListItem(EditPhoneNo.Text, cDefine.PHONE_NO_WAITING, "", "");
        }

        private void TimerAutoDialer_Tick(object sender, EventArgs e)
        {
            if (listView_PhoneNo.Items.Count == 0)
                return;

            if (m_objServerExCOM.GetCallCount() >= m_nConcurrentCallCount)
                return;

            foreach (ListViewItem LVitem in listView_PhoneNo.Items)
            {
                if (LVitem.SubItems[1].Text != cDefine.PHONE_NO_WAITING)
                    continue;

                string sPhoneNo = LVitem.SubItems[0].Text;

                if (sPhoneNo.Length == 0)
                    break;

                m_objServerExCOM.DialCall(sPhoneNo, ((cListViewItemEx)LVitem).m_nPhoneNoId);
                break;
            }

        }

        private void btn_LoadCSV_Click(object sender, EventArgs e)
        {
            openFileDlgCSV.Filter = "CSV Files |*.csv";
            DialogResult result = openFileDlgCSV.ShowDialog();
            
            if (result != DialogResult.OK)
            {
                return;
            }

            string sLoadFileName = openFileDlgCSV.FileName;

            if (sLoadFileName.Length == 0)
                return;

            List<string> objListPhoneNo = new List<string>();
            List<string> objListPhoneStatus = new List<string>();
            List<string> objListDTMF = new List<string>();
            List<string> objListDuration = new List<string>();

            cFileCSV objFileCSV = new cFileCSV(sLoadFileName);

            if (!objFileCSV.GetDataCSV(ref objListPhoneNo, ref objListPhoneStatus, ref objListDTMF, ref objListDuration))
                return;

            if (objListPhoneNo.Count != objListPhoneStatus.Count)
                return;

            for (int nIndex = 0; nIndex < objListPhoneNo.Count; nIndex++)
            {
                InsertListItem(objListPhoneNo[nIndex], objListPhoneStatus[nIndex], objListDTMF[nIndex], objListDuration[nIndex]);
            }
        }

        private void btn_RemoveListRecord_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem LVitem in listView_PhoneNo.SelectedItems)
            {
                m_objServerExCOM.CloseCall(((cListViewItemEx)LVitem).m_nPhoneNoId);
                listView_PhoneNo.Items.Remove(LVitem);
            }
        }

        private void btn_SaveDialerResult_Click(object sender, EventArgs e)
        {
            saveFileDlgCSV.Filter = "CSV Files |*.csv";
            saveFileDlgCSV.FileName = "PhoneNo.CSV";
            DialogResult result = saveFileDlgCSV.ShowDialog();

            if (result != DialogResult.OK)
                return;

            string sFileNameCSV = saveFileDlgCSV.FileName;

            if (sFileNameCSV.Length == 0)
                return;

            List<string> objListPhoneNo = new List<string>();
            List<string> objListPhoneStatus = new List<string>();
            List<string> objListDTMF = new List<string>();
            List<string> objListDuration = new List<string>();

            foreach (ListViewItem LVitem in listView_PhoneNo.Items)
            {
                if (LVitem.SubItems[0].Text.Length == 0)
                    LVitem.SubItems[0].Text = "  ";

                if (LVitem.SubItems[1].Text.Length == 0)
                    LVitem.SubItems[1].Text = "  ";

                if (LVitem.SubItems[2].Text.Length == 0)
                    LVitem.SubItems[2].Text = "  ";

                if (LVitem.SubItems[3].Text.Length == 0)
                    LVitem.SubItems[3].Text = "  ";

                objListPhoneNo.Add(LVitem.SubItems[0].Text);
                objListPhoneStatus.Add(LVitem.SubItems[1].Text);
                objListDTMF.Add(LVitem.SubItems[2].Text);
                objListDuration.Add(LVitem.SubItems[3].Text);
            }

            cFileCSV objFileCSV = new cFileCSV(sFileNameCSV);

            if (!objFileCSV.WriteDataCSV(objListPhoneNo, objListPhoneStatus, objListDTMF, objListDuration))
                return;
        }

        private void btn_LoadWave_Click(object sender, EventArgs e)
        {
            openFileDlgCSV.Filter = "Wav Files |*.wav";
            DialogResult result = openFileDlgCSV.ShowDialog();
            
            if (result != DialogResult.OK)
                 return;
            
            EditWaveFileDialer.Text = openFileDlgCSV.FileName;
        }

        private void check_DiagnosticLog_CheckedChanged(object sender, EventArgs e)
        {
            if (check_DiagnosticLog.Checked)
                m_objServerExCOM.DiagnosticLogSIP(true, true);
            else
                m_objServerExCOM.DiagnosticLogSIP(false, false);
        }

        private void check_DetectAMD_CheckedChanged(object sender, EventArgs e)
        {
            m_objServerExCOM.EnableDetectAMD(check_DetectAMD.Checked);
        }

        private void check_DetectDTMF_CheckedChanged(object sender, EventArgs e)
        {
            m_objServerExCOM.EnableDetectDTMF(check_DetectDTMF.Checked);
        }

        private void BtnDemoAccount_Click(object sender, EventArgs e)
        {
            TimeSpan EpocTime = DateTime.UtcNow - new DateTime(1970, 1, 1);
            int nEpocSeconds = ((int)EpocTime.TotalSeconds) % 9999999;

            String sAccountDemo = nEpocSeconds.ToString();
                        
            EditAuthName.Text = sAccountDemo;
            EditPassword.Text = sAccountDemo;

            EditDisplayName.Text = sAccountDemo;
            EditUserName.Text = sAccountDemo;

            EditDomain.Text = "demo.vaxvoip.com";

            EditServerAddr.Text = "demo.vaxvoip.com";
            EditServerPort.Text = "8060";
                        
            String sDemoMsg = "";

            sDemoMsg = "\n" + "Auth Login: " + sAccountDemo + "\n";
            sDemoMsg = sDemoMsg + "Auth Password: " + sAccountDemo + "\n";
            sDemoMsg = sDemoMsg + "Server Addr: " + EditServerAddr.Text + "\n";
            sDemoMsg = sDemoMsg + "Server Port: " + EditServerPort.Text + "\n \n";

            MessageBox.Show(sDemoMsg + "Demo accounts are only for account-to-account VoIP calls. Use demo accounts on different devices to dial/receive VoIP calls.\n\nDial 000 for self testing.\n\n[Powered by VaxVoIP Server SDK]", "VaxVoIP SDK", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
    }
 }
