﻿using System;
using System.Collections;
using Microsoft.Win32;

namespace AutoDialer
{
    class cStoreRegistry
    {
        const string REGISTRY_ROOT_PATH = "SOFTWARE\\VaxAutoDialer";

        private RegistryKey m_hKey;

        public cStoreRegistry()
        {
            Open();
        }

        ////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////

        private void Open()
        {
            if (m_hKey != null)
                return;

            m_hKey = Registry.CurrentUser.CreateSubKey(REGISTRY_ROOT_PATH);
        }

        public void Close()
        {
            if (m_hKey != null)
            {
                m_hKey.Close();
                m_hKey = null;
            }
        }

        ////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////

        private RegistryKey CreateKey(string sKeyName)
        {
            if (m_hKey == null)
                return null;

            if (sKeyName.Length == 0)
                return null;

            return m_hKey.CreateSubKey(sKeyName);
        }

        private RegistryKey OpenKey(string sKeyName, Boolean bWritable)
        {
            if (m_hKey == null)
                return null;

            if (sKeyName.Length == 0)
                return null;

            return m_hKey.OpenSubKey(sKeyName, bWritable);
        }

        //////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////

        public Boolean SetTEXT(string sKeyName, string sValueName, string sValueData)
        {
            if (sValueName.Length == 0) return false;

            RegistryKey hKey = CreateKey(sKeyName);
            if (hKey == null) return false;

            hKey.SetValue(sValueName, sValueData, RegistryValueKind.String);
            hKey.Close();

            return true;
        }

        public Boolean SetINT(string sKeyName, string sValueName, int nValueData)
        {
            if (sValueName.Length == 0) return false;

            RegistryKey hKey = CreateKey(sKeyName);
            if (hKey == null) return false;

            hKey.SetValue(sValueName, nValueData, RegistryValueKind.DWord);
            hKey.Close();

            return true;
        }

        public Boolean SetBOOL(string sKeyName, string sValueName, bool bValueData)
        {
            int nValueData = bValueData ? 1 : 0;

            return this.SetINT(sKeyName, sValueName, nValueData);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public void GetTEXT(string sKeyName, string sValueName, ref string sValueData, string sValueDefault)
        {
            if (sValueName.Length == 0)
            {
                sValueData = sValueDefault;
                return;
            }

            RegistryKey hKey = OpenKey(sKeyName, false);
            if (hKey == null)
            {
                sValueData = sValueDefault;
                return;
            }

            sValueData = (string) hKey.GetValue(sValueName, sValueDefault);
            hKey.Close();
        }

        public Boolean GetINT(string sKeyName, string sValueName, ref int nValueData, int nValueDefault)
        {
            if (sValueName.Length == 0)
            {
                nValueData = nValueDefault;
                return false;
            }

            RegistryKey hKey = OpenKey(sKeyName, false);

            if (hKey == null)
            {
                nValueData = nValueDefault;
                return false;
            }

            nValueData = (int)hKey.GetValue(sValueName, nValueDefault);
            hKey.Close();

            return true;
        }
        
        public void GetBOOL(string sKeyName, string sValueName, ref bool bValueData, bool bValueDefault)
        {
            int nValueData = 0;
            int nValueDefault = bValueDefault ? 1 : 0;

            if (!GetINT(sKeyName, sValueName, ref nValueData, nValueDefault))
            {
                bValueData = bValueDefault;
                return;
            }

            if (nValueData == 0)
                bValueData = false;
            else
                bValueData = true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public void DeleteSubKey(string sKeyName)
        {
            if (sKeyName.Length == 0)
                return;

            m_hKey.DeleteSubKeyTree(sKeyName, false);
        }

        public bool DeleteValue(string sKeyRootName, string sValueName)
        {
            if (sValueName.Length == 0)
                return false;

            RegistryKey hKey = OpenKey(sKeyRootName, true);
            if (hKey == null) return false;

            hKey.DeleteValue(sValueName, false);
            hKey.Close();

            return true;
        }

        public Boolean EnumSubKeys(string sKeyRootName, ref ArrayList objList)
        {
            RegistryKey hKey = OpenKey(sKeyRootName, false);
            if (hKey == null) return false;

            string[] arrayLogin = (string[])hKey.GetSubKeyNames();

            for (int nCount = 0; nCount < arrayLogin.Length; nCount++)
                objList.Add(arrayLogin[nCount]);

            hKey.Close();

            return true;
        }

        public bool IsKeyExist(string sKeyName)
        {
            RegistryKey hKey = OpenKey(sKeyName, false);
            if (hKey == null) return false;

            hKey.Close();
            return true;
        }
    }
}
