﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace BasicIVR
{

    public partial class MainForm : Form
    {
        cVaxServerExCOM m_objServerExCOM;
        UInt32 m_nPhoneNoId;
                                                
        public MainForm()
        {
            m_objServerExCOM = new cVaxServerExCOM(this);

            DateTime epoch = new DateTime(1970, 1, 1);
            DateTime obj = epoch.AddSeconds(8);
                        
            String sTime = String.Format("{0}:{1}:{2}", obj.Hour, obj.Minute, obj.Second);

            m_nPhoneNoId = 0;
            
            InitializeComponent();

            int nLineType = cVaxServerCOM.VAX_LINE_TYPE_UDP;

            string sDisplayName = "", sUserName = "";
            string sAuthUser = "", sAuthPwd = "";
            String sDomainRealm = "", sServerAddr = "";

            int nServerPort = cVaxServerCOM.SIP_DEFAULT_PORT;

            cStoreLine.GetLine(cVaxServerExCOM.VAX_ACCOUNT_LINE_SIP, ref nLineType, ref sDisplayName, ref sUserName, ref sAuthUser, ref sAuthPwd, ref sDomainRealm, ref sServerAddr, ref nServerPort);

            EditDisplayName.Text = sDisplayName;
            EditUserName.Text = sUserName;
            
            EditAuthName.Text = sAuthUser;
            EditPassword.Text = sAuthPwd;
            
            EditDomain.Text = sDomainRealm;
            EditServerAddr.Text = sServerAddr;
            EditServerPort.Text = Convert.ToString(nServerPort);
            
            int nListenPortSIP = cStoreCOMM.GetListenPortSIP();
            EditListenPort.Text = nListenPortSIP.ToString();
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void btnStartServer_Click(object sender, EventArgs e)
        {
            Boolean bStatusRun = m_objServerExCOM.Initialize(Convert.ToInt32(EditListenPort.Text));

            if (bStatusRun == false)
            {
                LabelServerStatus.Text = "UNABLE TO START SIP SERVER IVR (SEE ERROR LOG).";
                return;
            }

            if (check_DiagnosticLog.Checked)
                m_objServerExCOM.DiagnosticLogSIP(true, true);

            m_objServerExCOM.LoadMusic(EditPlayWaveFile.Text);
            cStoreCOMM.SetListenPortSIP(Convert.ToInt32(EditListenPort.Text));

            TimerCallCount.Enabled = true;

            LabelServerStatus.Text = "SIP SERVER IVR IS RUNNING.";
        }
                
        private void btnStop_Click(object sender, EventArgs e)
        {
            TimerCallCount.Enabled = false;
            m_objServerExCOM.UnInitialize();

            LabelServerStatus.Text = "SIP SERVER IVR IS NOT RUNNING.";
            LabelTotalCalls.Text = "0";

            btnRegisterAccountSIP.Text = "Register";
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void btnRegisterAccountSIP_Click(object sender, EventArgs e)
        {
            if (!m_objServerExCOM.IsInitialized())
            {
                MessageBox.Show("Please click the start button first.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            
            if(m_objServerExCOM.IsAddedAccountSIP())
            {
                m_objServerExCOM.UnRegisterAccountSIP();
                btnRegisterAccountSIP.Text = "Register";
                return;
            }

            if (!m_objServerExCOM.RegisterAccountSIP(EditDisplayName.Text, EditUserName.Text, EditAuthName.Text, EditPassword.Text, EditDomain.Text, EditServerAddr.Text, (Convert.ToInt32(EditServerPort.Text))))
                return;

            cStoreLine.AddLine(cVaxServerExCOM.VAX_ACCOUNT_LINE_SIP, cVaxServerCOM.VAX_LINE_TYPE_UDP, EditDisplayName.Text, EditUserName.Text, EditAuthName.Text, EditPassword.Text, EditDomain.Text, EditServerAddr.Text, Convert.ToInt32(EditServerPort.Text));
            btnRegisterAccountSIP.Text = "UnRegister";
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void TimerGeneral_Tick(object sender, EventArgs e)
        {
            LabelTotalCalls.Text = m_objServerExCOM.GetCallCount().ToString();
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public void OnVaxStatusLog(string sMsg)
        {
            ListStatus.Items.Add(sMsg);
            ListStatus.TopIndex = ListStatus.Items.Count - 1;
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
     
        public void OnPhoneNoStatusLog(uint nPhoneNoId, string sPhoneNo, string sStatus, string sDigitDTMF, string sDuration, string sRecordFile)
        {
            UpdateListItem(nPhoneNoId, sPhoneNo, sStatus, sDigitDTMF, sDuration, sRecordFile);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void btnClearLog_Click(object sender, EventArgs e)
        {
            ListStatus.Items.Clear();
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public uint InsertListItem(string sPhoneNo, string sPhoneStatus, string sDigitDTMF, string sDuration, string sRecordFile)
        {
            sPhoneNo.Trim();
            sPhoneStatus.Trim();

            m_nPhoneNoId++;

            listView_PhoneNo.Items.Add(new cListViewItemEx(m_nPhoneNoId, sPhoneNo, sPhoneStatus, sDigitDTMF, sDuration, sRecordFile));
            listView_PhoneNo.TopItem = listView_PhoneNo.Items[listView_PhoneNo.Items.Count - 1];

            return m_nPhoneNoId;
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void UpdateListItem(uint nPhoneNoId, string sPhoneNo, string sPhoneStatus, string sDigitDTMF, string sDuration, string sRecordFile)
        {
            foreach (ListViewItem LVitem in listView_PhoneNo.Items)
            {
                if (((cListViewItemEx)LVitem).m_nPhoneNoId == nPhoneNoId)
                {
                    if (sPhoneStatus.Length != 0)
                        LVitem.SubItems[1].Text = sPhoneStatus;

                    if (sDigitDTMF.Length != 0)
                        LVitem.SubItems[2].Text = sDigitDTMF;

                    if (sDuration.Length != 0)
                        LVitem.SubItems[3].Text = sDuration;

                    if (sRecordFile.Length != 0)
                        LVitem.SubItems[4].Text = sRecordFile;

                    break;
                }
            }
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void btn_LoadCSV_Click(object sender, EventArgs e)
        {
            openFileDlgCSV.Filter = "CSV Files |*.csv";
            DialogResult result = openFileDlgCSV.ShowDialog();

            if (result != DialogResult.OK)
            {
                return;
            }

            string sLoadFileName = openFileDlgCSV.FileName;

            if (sLoadFileName.Length == 0)
                return;

            List<string> objListPhoneNo = new List<string>();
            List<string> objListPhoneStatus = new List<string>();
            List<string> objListDTMF = new List<string>();
            List<string> objListDuration = new List<string>();
            List<string> objListRecordFile = new List<string>();

            cFileCSV objFileCSV = new cFileCSV(sLoadFileName);

            if (!objFileCSV.GetDataCSV(ref objListPhoneNo, ref objListPhoneStatus, ref objListDTMF, ref objListDuration, ref objListRecordFile))
                return;

            if (objListPhoneNo.Count != objListPhoneStatus.Count)
                return;

            for (int nIndex = 0; nIndex < objListPhoneNo.Count; nIndex++)
            {
                InsertListItem(objListPhoneNo[nIndex], objListPhoneStatus[nIndex], objListDTMF[nIndex], objListDuration[nIndex], objListRecordFile[nIndex]);
            }
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void btn_RemoveListRecord_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem LVitem in listView_PhoneNo.SelectedItems)
            {
                m_objServerExCOM.CloseCall(((cListViewItemEx)LVitem).m_nPhoneNoId);
                listView_PhoneNo.Items.Remove(LVitem);
            }
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void btn_SaveResult_Click(object sender, EventArgs e)
        {
            saveFileDlgCSV.Filter = "CSV Files |*.csv";
            saveFileDlgCSV.FileName = "PhoneNo.CSV";
            DialogResult result = saveFileDlgCSV.ShowDialog();

            if (result != DialogResult.OK)
                return;

            string sFileNameCSV = saveFileDlgCSV.FileName;

            if (sFileNameCSV.Length == 0)
                return;

            List<string> objListPhoneNo = new List<string>();
            List<string> objListPhoneStatus = new List<string>();
            List<string> objListDTMF = new List<string>();
            List<string> objListDuration = new List<string>();
            List<string> objListRecordFile = new List<string>();

            foreach (ListViewItem LVitem in listView_PhoneNo.Items)
            {
                if (LVitem.SubItems[0].Text.Length == 0)
                    LVitem.SubItems[0].Text = "  ";

                if (LVitem.SubItems[1].Text.Length == 0)
                    LVitem.SubItems[1].Text = "  ";

                if (LVitem.SubItems[2].Text.Length == 0)
                    LVitem.SubItems[2].Text = "  ";

                if (LVitem.SubItems[3].Text.Length == 0)
                    LVitem.SubItems[3].Text = "  ";

                if (LVitem.SubItems[4].Text.Length == 0)
                    LVitem.SubItems[4].Text = "  ";

                objListPhoneNo.Add(LVitem.SubItems[0].Text);
                objListPhoneStatus.Add(LVitem.SubItems[1].Text);
                objListDTMF.Add(LVitem.SubItems[2].Text);
                objListDuration.Add(LVitem.SubItems[3].Text);
                objListRecordFile.Add(LVitem.SubItems[4].Text);
            }

            cFileCSV objFileCSV = new cFileCSV(sFileNameCSV);

            if (!objFileCSV.WriteDataCSV(objListPhoneNo, objListPhoneStatus, objListDTMF, objListDuration, objListRecordFile))
                return;
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void btn_LoadWave_Click(object sender, EventArgs e)
        {
            openFileDlgCSV.Filter = "Wav Files |*.wav";
            DialogResult result = openFileDlgCSV.ShowDialog();
            
            if (result != DialogResult.OK)
                 return;
            
            EditPlayWaveFile.Text = openFileDlgCSV.FileName;
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void check_DiagnosticLog_CheckedChanged(object sender, EventArgs e)
        {
            if (check_DiagnosticLog.Checked)
                m_objServerExCOM.DiagnosticLogSIP(true, true);
            else
                m_objServerExCOM.DiagnosticLogSIP(false, false);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
   
        private void check_DetectDTMF_CheckedChanged(object sender, EventArgs e)
        {
            m_objServerExCOM.EnableDetectDTMF(check_DetectDTMF.Checked);
        }

        private void check_PlayWave_CheckedChanged(object sender, EventArgs e)
        {
            m_objServerExCOM.SetPlayWaveConnected(check_PlayWave.Checked);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
    }
 }
