﻿using System;
using System.Text;
using System.IO;

namespace BasicIVR
{
    public class cVaxServerExCOM : cVaxServerCOM
    {
        public static cVaxServerExCOM mVaxExCOM;

        int m_nPlayMusic = -1;
        Boolean m_bInitialized = false;

        Boolean m_bIsLineAdded = false;

        Boolean m_bPlayWaveConnected = true;
        Boolean m_bDetectDTMF = true;

        cVaxCallSessionList m_objSessionList = null;
        private MainForm m_objMainForm = null;

        public const string VAX_ACCOUNT_LINE_SIP = "LineSIP";

        #region Methods

        public cVaxServerExCOM(MainForm objMainForm)
        {
            mVaxExCOM = this;

            m_objMainForm = objMainForm;
        }

        ~cVaxServerExCOM()
        {

        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////

        private void ResetAll()
        {
            m_objSessionList = new cVaxCallSessionList();
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////////////////////

        public bool Initialize(int nListenPort)
        {
            UnInitialize();

            base.SetLicenseKey("TRIAL-LICENSE-KEY");

            if (!base.Initialize(""))
                return false;

            if (!base.OpenNetworkUDP("", nListenPort))
            {
                UnInitialize();
                return false;
            }

            /*
            if (!base.OpenNetworkTCP("", nListenPort))
            {
                UnInitialize();
                return false;
            }

            if (!base.OpenNetworkTLS("", nListenPort + 1, ""))
            {
                UnInitialize();
                return false;
            }
            */

            // base.AttackDetectScanSIP("demo.vaxvoip.com");
            // base.AttackDetectFloodSIP(1000);
            // base.AttackDetectBruteForceSIP(10, 2);

            base.SetListenPortRangeRTP(20000, 40000);
            
            base.AudioSessionLost(true, 20); // Enable voice session lost for 20 seconds.
            base.VideoCOMM(false);

            for (int nCount = 4000; nCount <= 4080; nCount++)
            {
                base.AddUser(nCount.ToString(), nCount.ToString(), "32");
            }

            m_bInitialized = true;
            return true;
        }

        public new void UnInitialize()
        {
            m_bInitialized = false;
            UnLoadMusic();

            ResetAll();
           
            m_nPlayMusic = -1;
            m_bIsLineAdded = false;

            base.UnInitialize();
        }

        public bool IsInitialized()
        {
            return m_bInitialized;
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public bool RegisterAccountSIP(String sDisplayName, String sUserName, String sAuthUser, String sAuthPwd, String sDomainRealm, String sServerAddr, int nServerPort)
        {
            if (!base.AddLine(VAX_ACCOUNT_LINE_SIP, VAX_LINE_TYPE_UDP, sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sServerAddr, nServerPort, "32"))
                return false;

            if (!RegisterLine(VAX_ACCOUNT_LINE_SIP, 1800))
                return false;

            m_bIsLineAdded = true;

            return true;
        }

        public void UnRegisterAccountSIP()
        {
            base.UnRegisterLine(VAX_ACCOUNT_LINE_SIP);
            base.RemoveLine(VAX_ACCOUNT_LINE_SIP);

            m_bIsLineAdded = false;
        }

        public bool IsAddedAccountSIP()
        {
            return m_bIsLineAdded;
        }

        /////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////

        public void CloseCall(UInt32 nPhoneNoId)
        {
            cVaxCallSession objCallSession = m_objSessionList.FindPhoneNoSession(nPhoneNoId);
            if (objCallSession == null) return;


            CloseCallSession(objCallSession.m_nSessionId);
        }
                
        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public Boolean LoadMusic(string sWaveFile)
        {
            UnLoadMusic();

            if (sWaveFile.Length == 0)
                return false;

            // Load wave file just one-time, don't load it again and again, it affects the performance. Load multiple files on startup.

            m_nPlayMusic = base.LoadWaveFile(sWaveFile);

            if (m_nPlayMusic == -1)
                return false;

            return true;
        }

        public void UnLoadMusic()
        {
            if (m_nPlayMusic == -1)
                return;

            base.UnLoadWaveID(m_nPlayMusic);

            m_nPlayMusic = -1;
        }

        public void PlayMusic(int nSessionId)
        {
            if (m_nPlayMusic == -1)
                return;

            base.PlayWaveSetModeTypeToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, cVaxServerCOM.VAX_PLAY_WAVE_MODE_MIXED);
            base.PlayWaveStartToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, m_nPlayMusic, false, 10);
        }

        public void PlayText(int nSessionId, int nChannelId, String sText)
        {
            Byte[] aDataPCM = null;
            long nDataSize = 0;

            cTextToSpeech objSpeech = new cTextToSpeech();
            objSpeech.GenSpeech(sText, ref aDataPCM, ref nDataSize);

            base.SendAudioPCM(nSessionId, nChannelId, aDataPCM, (int) nDataSize);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public void EnableDetectDTMF(Boolean bDetectDTMF)
        {
            m_bDetectDTMF = bDetectDTMF;
        }

        public void SetPlayWaveConnected(Boolean bPlayWaveConnected)
        {
            m_bPlayWaveConnected = bPlayWaveConnected;
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        #endregion

        #region Events

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionCreated(int nSessionId, int nReasonCode)
        {
            m_objSessionList.AddSession(nSessionId);
        }

        protected override void OnCallSessionClosed(int nSessionId, int nChannelId, int nReasonCode)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            if (objCallSession.IsConnected())
            {
                objCallSession.EndCall();
                String sDuration = objCallSession.GetDuration();

                m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, "", "", sDuration, "");
            }

            objCallSession.OnCallSessionClosed(nSessionId, nChannelId, nReasonCode);

            m_objSessionList.RemoveSession(nSessionId);
        }

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        protected override void OnCallSessionAccessAudioPCM(int nSessionId, int nChannelId, int nAccessType, Byte[] aDataPCM, int nSizePCM, int nTypePCM)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionAccessAudioPCM(nSessionId, nAccessType, aDataPCM, nSizePCM);
        }

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnIncomingCall(int nSessionId, String sCallerName, String sCallerId, String sDialNo, int nFromPeerType, String sFromPeerName, String sRouteUID, int nRouteType, String sRouteName, String sUserAgentName, String sFromIP, int nFromPort)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            string sPhoneNo = sCallerName;

            if (sCallerName.Length == 0)
            {
                if (sCallerId.Length != 0)
                    sPhoneNo = sCallerId;
                else
                    sPhoneNo = "Unknown";
            }

            uint nPhoneNoId = m_objMainForm.InsertListItem(sPhoneNo, cDefine.PHONE_NO_PROCESSING, "", "", "");

            objCallSession.m_sPhoneNo = sPhoneNo;
            objCallSession.m_sStatus = cDefine.PHONE_NO_PROCESSING;

            objCallSession.m_nPhoneNoId = nPhoneNoId;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, cDefine.PHONE_NO_PROCESSING, "", "", "");

            base.AcceptCallSession(nSessionId, "", "", "", "", "", 20);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
                
        protected override void OnCallSessionConnected(int nSessionId)
        {
            if (m_bPlayWaveConnected)
                PlayMusic(nSessionId);

            CGenFileLocation objGenFileLoc = new CGenFileLocation();
            string sRecFilePath = objGenFileLoc.GenRecordingFilePath(nSessionId, "RecordFiles");

            RecordWaveStartToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, sRecFilePath, cVaxServerCOM.VAX_WAVE_REC_TYPE_MONO_PCM);

            if (m_bDetectDTMF)
            {
                base.DetectDigitDTMF(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, cVaxServerCOM.VAX_DTMF_TYPE_RFC2833, true, 1500);  // Enable DTMF detection RFC2833 based.
                base.DetectDigitDTMF(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, cVaxServerCOM.VAX_DTMF_TYPE_SIP_INFO, true, 1500);     // Enable DTMF detection SIP INFO based.
            }

            cVaxCallSession objCallSession;

            objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionConnected(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO);

            objCallSession.StartCall();
            objCallSession.SetConnected();

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, cDefine.PHONE_NO_CONNECTED, "", "", sRecFilePath);

        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnDetectedDigitDTMF(int nSessionId, int nChannelId, String sDigitDTMF, int nTypeDTMF)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, "", sDigitDTMF, "", "");

            string sMsgDTMF = "Digit Detected: [ " + sDigitDTMF + " ]";

            PlayText(nSessionId, nChannelId, sMsgDTMF);

            GenStatusLog(sMsgDTMF);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionFailed(int nSessionId, int nChannelId, int nStatusCode, String sReasonPhrase, String sContact)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            string sStatus;
            sStatus = cDefine.PHONE_NO_FAILED + " : " + nStatusCode.ToString() + "-" + sReasonPhrase;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, sStatus, "", "", "");
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionTimeout(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, cDefine.PHONE_NO_TIMEOUT, "", "", "");
        }

        protected override void OnCallSessionCancelled(int nSessionId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, cDefine.PHONE_NO_CANCELED, "", "", "");
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
        protected override void OnCallSessionMediaDetectStartedVAD(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectStartedVAD(nSessionId, nChannelId);
        }
        protected override void OnCallSessionMediaDetectCompletedVAD(int nSessionId, int nChannelId, Byte[] aDataPCM, int nSizePCM, int nTypePCM)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectCompletedVAD(nSessionId, nChannelId, aDataPCM, nSizePCM, nTypePCM);
        }

        protected override void OnCallSessionMediaDetectExpiredVAD(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectExpiredVAD(nSessionId, nChannelId);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnRegisterUser(string sUserLogin, string sDomain, string sUserAgentName, string sFromIP, int nFromPort, int nRegId)
        {
            base.AuthRegister(nRegId);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        protected override void OnLineRegisterTrying(string sLineName)
        {
            string sStatus = sLineName + " " + cDefine.LINE_REG_TRYING;
            GenStatusLog(sStatus);
        }

        protected override void OnLineRegisterFailed(string sLineName, int nStatusCode, string sReasonPhrase)
        {
            string sStatus = sLineName + " Register Failed: " + sReasonPhrase;
            GenStatusLog(sStatus);
        }

        protected override void OnLineRegisterSuccess(string sLineName)
        {
            string sStatus = sLineName + " " + cDefine.LINE_REG_SUCCESS;
            GenStatusLog(sStatus);
        }

        ///////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public void GenStatusLog(string sStatus)
        {
            m_objMainForm.OnVaxStatusLog(sStatus);
        }

        protected override void OnVaxErrorLog(String sFuncName, int nErrorCode, String sErrorMsg)
        {
            string sMsg = "[Method: " + sFuncName + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
            GenStatusLog(sMsg);
        }

        protected override void OnCallSessionErrorLog(int nSessionId, int nChannelId, int nErrorCode, String sErrorMsg)
        {
            string sMsg = "[Session: " + nSessionId.ToString() + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
            GenStatusLog(sMsg);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnOutgoingDiagnosticLog(string sMsgSIP, string sToIP, int nToPort)
        {
            FileStream LogFile = new FileStream("VaxLog.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite);
            LogFile.Seek(0, SeekOrigin.End);

            String sNewLine = "\r\n";
            byte[] aNewLine = Encoding.ASCII.GetBytes(sNewLine);
            LogFile.Write(aNewLine, 0, sNewLine.Length);

            String sAddressInfo = String.Format("{0}:{1}", sToIP, nToPort);
            byte[] aAddressInfo = Encoding.ASCII.GetBytes(sAddressInfo);
            LogFile.Write(aAddressInfo, 0, sAddressInfo.Length);

            LogFile.Write(aNewLine, 0, sNewLine.Length);

            String sLogType = "============== OUTGOING============";
            byte[] aLogType = Encoding.ASCII.GetBytes(sLogType);
            LogFile.Write(aLogType, 0, sLogType.Length);

            LogFile.Write(aNewLine, 0, sNewLine.Length);

            byte[] aMsgSIP = Encoding.ASCII.GetBytes(sMsgSIP);
            LogFile.Write(aMsgSIP, 0, sMsgSIP.Length);

            LogFile.Close();
        }

        protected override void OnIncomingDiagnosticLog(string sMsgSIP, string sFromIP, int nFromPort)
        {
            FileStream LogFile = new FileStream("VaxLog.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite);
            LogFile.Seek(0, SeekOrigin.End);

            String sNewLine = "\r\n";
            byte[] aNewLine = Encoding.ASCII.GetBytes(sNewLine);
            LogFile.Write(aNewLine, 0, sNewLine.Length);

            String sAddressInfo = String.Format("{0}:{1}", sFromIP, nFromPort);
            byte[] aAddressInfo = Encoding.ASCII.GetBytes(sAddressInfo);
            LogFile.Write(aAddressInfo, 0, sAddressInfo.Length);

            LogFile.Write(aNewLine, 0, sNewLine.Length);

            String sLogType = "============== INCOMING ============";
            byte[] aLogType = Encoding.ASCII.GetBytes(sLogType);
            LogFile.Write(aLogType, 0, sLogType.Length);

            LogFile.Write(aNewLine, 0, sNewLine.Length);

            byte[] aMsgSIP = Encoding.ASCII.GetBytes(sMsgSIP);
            LogFile.Write(aMsgSIP, 0, sMsgSIP.Length);

            LogFile.Close();
        }


        #endregion

    }
}
