unit VaxPipeClientThread;

interface

uses
  System.Classes, Windows, System.SysUtils,
  System.Threading, VaxPipeClient;


type TVaxPipeClientThread = class(TThread)

  private
  m_sPipeNameZERO : String;
  m_sPipeNameONE : String;

  private
  m_hQuitWait : THandle;
  m_bQuitThread : Boolean;

  public
  constructor Create(sPipeNameZERO: String; sPipeNameONE: String);
  destructor Destroy(); override;

  function PostWritePipe(pData: PByte; nSize: Integer): Boolean;

  private
  procedure OnPipeData(nMediaPos: Integer; aDataReadPipe: PByte; nSizeReadPipe: Integer);
  procedure OnThreadMsg(objPipeClient : TVaxPipeClient);

  public
  procedure OnVaxPipeClientRecvPCM(nMediaPos: Integer; aDataPCM: PByte; nSizePCM: Integer); virtual;

  public
  procedure Execute; override;

end;

implementation

const
  VAX_SYSTEM_THREAD_MSG_WRITE_PIPE = 102;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  constructor TVaxPipeClientThread.Create(sPipeNameZERO: String; sPipeNameONE: String);
  begin

    m_bQuitThread := false;

    m_sPipeNameZERO := sPipeNameZERO;
    m_sPipeNameONE := sPipeNameONE;

    m_hQuitWait := CreateEvent(NIL, true, false, NIL);

    inherited Create();

  end;

  destructor TVaxPipeClientThread.Destroy();
  var
    hQuitWait : THandle;
  begin

    if(m_hQuitWait = 0) Then
    begin
      Exit();
    end;

    hQuitWait := m_hQuitWait;
    m_hQuitWait := 0;

    m_bQuitThread := true;

    SetEvent(hQuitWait);
    Inherited WaitFor();

    CloseHandle(hQuitWait);
    inherited;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxPipeClientThread.Execute;
  var
    aDataReadPipe : PByte;
    nSizeReadPipe : Integer;

    aWaitEvent : Array[0..4] of THandle;
    aPipeClient : Array[0..2] of TVaxPipeClient;

    nWaitCount : Integer;
    nWaitIndex : LongWord;
  begin

    ////////////////////////////////////////////////////////////////////////////

    aPipeClient[0] := TVaxPipeClient.Create();

    if aPipeClient[0].CreatePipeReadWrite(m_sPipeNameZERO, aWaitEvent[0], aWaitEvent[2]) = false then
      Exit();


    aPipeClient[1] := TVaxPipeClient.Create();

    if aPipeClient[1].CreatePipeReadOnly(m_sPipeNameONE, aWaitEvent[1]) = false then
      Exit();

    ////////////////////////////////////////////////////////////////////////////

    if(aPipeClient[0].BeginReadPipe() = false) Then
    begin
      aPipeClient[0].ClosePipe();
      Exit();
    end;

    if(aPipeClient[1].BeginReadPipe() = false) Then
    begin
      aPipeClient[1].ClosePipe();
      Exit();
    end;

    ////////////////////////////////////////////////////////////////////////////

    aWaitEvent[3] := m_hQuitWait;
    nWaitCount := 4;

    ////////////////////////////////////////////////////////////////////////////

    While true do
    begin

    ////////////////////////////////////////////////////////////////////////////

      nWaitIndex := MsgWaitForMultipleObjects(nWaitCount, aWaitEvent, False, INFINITE, QS_POSTMESSAGE);

      if((nWaitIndex = WAIT_FAILED) OR (m_bQuitThread = true)) Then
        break;

    ////////////////////////////////////////////////////////////////////////////

      if (WAIT_OBJECT_0 + nWaitIndex) >= nWaitCount then
      begin
        OnThreadMsg(aPipeClient[0]);
        Continue;
      end;

      if ((WAIT_OBJECT_0 + nWaitIndex) = 3) Then
      begin
        break;
      end;

      if ((WAIT_OBJECT_0 + nWaitIndex) = 2) Then
      begin

        aPipeClient[0].EndWritePipe();

        Continue;

      end;

      if ((WAIT_OBJECT_0 + nWaitIndex) = 0) Then
      begin

        if(aPipeClient[0].EndReadPipe(aDataReadPipe, nSizeReadPipe) = false) Then
          Break;

        OnPipeData(0, aDataReadPipe, nSizeReadPipe);

        if(aPipeClient[0].BeginReadPipe() = false) Then
          Break;

        Continue;

      end;

      if ((WAIT_OBJECT_0 + nWaitIndex) = 1) Then
      begin

        if(aPipeClient[1].EndReadPipe(aDataReadPipe, nSizeReadPipe) = false) Then
          Break;

        OnPipeData(1, aDataReadPipe, nSizeReadPipe);

        if(aPipeClient[1].BeginReadPipe() = false) Then
          Break;

        Continue;

      end;


    ////////////////////////////////////////////////////////////////////////////

    end;

    aPipeClient[0].ClosePipe();
    aPipeClient[1].ClosePipe();

    aPipeClient[0].Free();
    aPipeClient[1].Free();

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxPipeClientThread.OnPipeData(nMediaPos: Integer; aDataReadPipe: PByte; nSizeReadPipe: Integer);
  begin
    OnVaxPipeClientRecvPCM(nMediaPos, aDataReadPipe, nSizeReadPipe);
  end;

  procedure TVaxPipeClientThread.OnVaxPipeClientRecvPCM(nMediaPos: Integer; aDataPCM: PByte; nSizePCM: Integer);
  begin

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxPipeClientThread.OnThreadMsg(objPipeClient : TVaxPipeClient);
  var
    hMsg: TMsg;
    pData: PByte;
    nSize: Integer;
  begin

    FillChar(hMsg, SizeOf(hMsg), 0);

    if not PeekMessage(hMsg, 0, 0, 0, PM_REMOVE) then
    begin
      Exit;
    end;

    if (hMsg.message = VAX_SYSTEM_THREAD_MSG_WRITE_PIPE) and (hMsg.hwnd = 0) then
    begin

      pData := PByte(hMsg.wParam);
      nSize := Integer(hMsg.lParam);

      objPipeClient.BeginWritePipe(pData, nSize);
      FreeMem(pData);

    end;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TVaxPipeClientThread.PostWritePipe(pData: PByte; nSize: Integer): Boolean;
  var
    pMsgData: PByte;
  begin

    GetMem(pMsgData, nSize);
    Move(pData^, pMsgData^, nSize);

    if not PostThreadMessage(ThreadId, VAX_SYSTEM_THREAD_MSG_WRITE_PIPE, WPARAM(pMsgData), LPARAM(nSize)) then
    begin
      FreeMem(pMsgData);
      Exit(False);
    end;

    Exit(True);

  end;


end.
