unit VaxLine;

Interface

uses

  System.Classes,
  StoreLine,
  VaxCallSession;


type TVaxLine = class

  public
  class procedure OnLineRegisterTrying(sLineName: string); static;
  class procedure OnLineRegisterFailed(sLineName: string; nStatusCode: Integer; sReasonPhrase: string); static;
  class procedure OnLineRegisterSuccess(sLineName: string); static;

  public
  class procedure RemoveLine(sLineName: String); static;
  class function AddLine(sLineName: String; nLineType: Integer; sAuthUser: String; sAuthPwd: String; sServerAddr: String; nServerPort: Integer; sAudioCodecList: String; bRegister: Boolean; sToPeerName: String; bRejected: Boolean; sRoomName: String): Boolean; static;
  class procedure LoadLine(); static;

end;

implementation

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  Uses VaxServerSIP, VaxServerCOM;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class procedure TVaxLine.OnLineRegisterTrying(sLineName: string);
  var
    sStatus: string;
  begin
    sStatus := sLineName + ': Trying to register';
    TVaxServerSIP.mVaxSIP.GenStatusLog(sStatus);
  end;

  class procedure TVaxLine.OnLineRegisterFailed(sLineName: string; nStatusCode: Integer; sReasonPhrase: string);
  var
    sStatus: string;
  begin
    sStatus := sLineName + ': Failed to register (' + sReasonPhrase + ')';
    TVaxServerSIP.mVaxSIP.GenStatusLog(sStatus);
  end;

  class procedure TVaxLine.OnLineRegisterSuccess(sLineName: string);
  var
    sStatus: string;
  begin
    sStatus := sLineName + ': Registered successfully';
    TVaxServerSIP.mVaxSIP.GenStatusLog(sStatus);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class function TVaxLine.AddLine(sLineName: String; nLineType: Integer; sAuthUser: String; sAuthPwd: String; sServerAddr: String; nServerPort: Integer; sAudioCodecList: String; bRegister: Boolean; sToPeerName: String; bRejected: Boolean; sRoomName: String): Boolean;
  begin

    RemoveLine(sLineName);

    if((TVaxServerSIP.mVaxSIP.IsInitialized() = False) or (Length(sLineName) = 0)) Then
      Exit(False);

    if(TVaxServerSIP.mVaxSIP.AddLine(sLineName, nLineType, '', '', sAuthUser, sAuthPwd, '', sServerAddr, nServerPort, sAudioCodecList) = False) Then
      Exit(False);

    TVaxCallSession.AddLineRoute(sLineName, sToPeerName, bRejected, sRoomName);

    if(bRegister = True) Then
      TVaxServerSIP.mVaxSIP.RegisterLine(sLineName, 1800);

    TStoreLine.AddLine(sLineName, nLineType, sAuthUser, sAuthPwd, sServerAddr, nServerPort, sAudioCodecList, bRegister, sToPeerName, bRejected, sRoomName);
    Result := True;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class procedure TVaxLine.RemoveLine(sLineName: String);
  begin

    if((TVaxServerSIP.mVaxSIP.IsInitialized() = False) or (Length(sLineName) = 0)) Then
      Exit();

    TVaxServerSIP.mVaxSIP.RemoveLine(sLineName);
    TVaxCallSession.RemoveLineRoute(sLineName);

    TStoreLine.RemoveLine(sLineName);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class procedure TVaxLine.LoadLine();
  var
    sLineName, sAuthUser, sAuthPwd, sServerAddr, sAudioCodecList: string;
    sToPeerName, sRoomName: string;

    nServerPort, nLineType: Integer;
    bRegister, bRejected: boolean;

    objListLines: TStringList;
  begin

    if(TVaxServerSIP.mVaxSIP.IsInitialized() = False) Then
      Exit();

    objListLines :=  TStringList.Create();
    TStoreLine.GetLineAll(objListLines);

    if(objListLines.Count = 0) Then
      Exit();

     sAuthUser := ''; sAuthPwd := ''; sServerAddr := ''; sAudioCodecList := '';
     sToPeerName := ''; sRoomName := '';

     bRejected := False; bRegister := False;
     nLineType := TVaxServerCOM.VAX_LINE_TYPE_UDP; nServerPort := TVaxServerCOM.SIP_DEFAULT_PORT;

    for sLineName in objListLines do
    begin
      TStoreLine.GetLine(sLineName, nLineType, sAuthUser, sAuthPwd, sServerAddr, nServerPort, sAudioCodecList, bRegister, sToPeerName, bRejected, sRoomName);
      AddLine(sLineName, nLineType, sAuthUser, sAuthPwd, sServerAddr, nServerPort, sAudioCodecList, bRegister, sToPeerName, bRejected, sRoomName);
    end;

  end;

end.
