unit VaxServerSIP;

Interface

uses

    System.Variants, System.SysUtils, System.Classes,
    StatusLogDlg, SIPRecDlg,

    VaxServerCOM;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

type TVaxServerSIP = class(TVaxServerCOM)

  public
  class var mVaxSIP: TVaxServerSIP;

  public
  m_objStatusLogDlg : TStatusLogForm;
  m_objRecDlgSIP : TSIPRecForm;

  //////////////////////////////////////////////////////////////////////////////

  private
  m_bIsActivatedREC: Boolean;

  m_nDialToneId: Integer;
  m_bInitialized: Boolean;

  private
  const LINE_NAME_REC = 'LineServerREC';

  //////////////////////////////////////////////////////////////////////////////

  private
  procedure ResetAll();

  public
  constructor Create();

  public
  function StartServer(nListenPort: Integer): boolean;
  procedure StopServer();

  function IsInitialized(): Boolean;

  ///////////////////////////////////////////////////////////////////////////////////////////

  function AcceptCallSession(nSessionId: Integer; sCallerName: String; sCallerId: String; sDialNo: String; sToPeerName: String; sRouteUID: String): Boolean;

  public
  function IsLineREC(sLineName: String): Boolean;
  procedure DeactivateREC;
  function ActivateREC(sAuthLogin: String; sAuthPwd: String; sServerAddr: String; nServerPort: Integer; bRegister: Boolean): Boolean;
  function IsActivatedREC(): Boolean;

  ///////////////////////////////////////////////////////////////////////////////////////////

  public
  function DialToneToCallSession(bEnable: Boolean; nSessionId: Integer) : Boolean;
  procedure LoadMusicFiles(sMusicHold: String; sMusicDialTone: String; sMusicCallPark: String);

  private
  function LoadMusicDialTone(sFileName: String) : Boolean;

  ///////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////

  public
  procedure OnCallSessionCreated(nSessionId: Integer; nReasonCode: Integer); override;
  procedure OnCallSessionClosed(nSessionId: Integer; nChannelId: Integer; nReasonCode: Integer); override;

  procedure OnCallSessionAccessAudioPCM(nSessionId: Integer; nChannelId: Integer; nAccessType: Integer; pDataPCM: PByte; nSizePCM: Integer; nTypePCM: Integer); override;

  procedure OnLineRegisterTrying(sLineName: String); override;
  procedure OnLineRegisterFailed(sLineName: String; nStatusCode: Integer; sReasonPhrase: String); override;
  procedure OnLineRegisterSuccess(sLineName: String); override;

  procedure OnUnRegisterUser(sUserLogin: String); override;
  procedure OnRegisterUser(sUserLogin: String; sDomain: String; sUserAgentName: String; sFromIP: String; nFromPort: Integer; nRegId: Integer); override;
  procedure OnRegisterUserSuccess(sUserLogin: String; sFromIP: String; nFromPort: Integer; nRegId: Integer); override;
  procedure OnRegisterUserFailed(sUserLogin: String; sFromIP: String; nFromPort: Integer; nRegId: Integer); override;

  procedure OnChatMessageText(nChatMsgId: Integer; sMsgFrom: String; sMsgTo: String; sMsgText: String; nFromPeerType: Integer; sFromPeerName: String; sFromIP: String; nFromPort: Integer); override;
  procedure OnChatMessageTyping(nChatMsgId: Integer; sMsgFrom: String; sMsgTo: String; bIsTypingStart: Boolean; nFromPeerType: Integer; sFromPeerName: String; sFromIP: String; nFromPort: Integer); override;
  procedure OnChatStatusSubscribe(nSubscrbId: Integer; sMsgFrom: String; sMsgTo: String; nFromPeerType: Integer; sFromPeerName: String; sFromIP: String; nFromPort: Integer); override;

  procedure OnCallSessionPushSIP(nSessionId: Integer; nChannelId: Integer; sCallerName: String; sCallerId: String; nProviderPN: Integer; sProjectPN: String; sDevicePN: String); override;

  procedure OnIncomingCall(nSessionId: Integer; sCallerName: String; sCallerId: String; sDialNo: String; nFromPeerType: Integer; sFromPeerName: String; sRouteUID: String; nRouteType: Integer; sRouteName: String; sUserAgentName: String; sFromIP: String; nFromPort: Integer); override;
  procedure OnCallSessionTransferBlind(nTransfererSessionId: Integer; nTransfererChannelId: Integer; sTransferer: String; sTransferee: String; sTransferTo: String; sRouteUID: String; nRouteType: Integer; sRouteName: String); override;
  procedure OnCallSessionTransferConsult(nTransfererSessionId: Integer; nTransfererChannelId: Integer; nTransferToSessionId: Integer; nTransferToChannelId: Integer; sTransferer: String; sTransferee: String; sTransferTo: String); override;

  procedure OnBusyLampSubscribe(nSubscrbId: Integer; sUserName: String; sToUserName: String; sFromIP: String; nFromPort: Integer); override;
  procedure OnBusyLampSendStatus(sFromUserName: String; sToUserName: String; nStateId: Integer); override;

  procedure OnDetectedDigitDTMF(nSessionId: Integer; nChannelId: Integer; sDigitDTMF: String; nTypeDTMF: Integer); override;
  procedure OnCallSessionDetectedAMD(nSessionId: Integer; nChannelId: Integer; bIsHuman: Boolean); override;

  procedure OnStealthListenerAttached(nAttachType: Integer; nStealthSessionId: Integer; nAttachSessionId: Integer; nAttachChannelId: Integer); override;

  procedure OnCallSessionErrorLog(nSessionId: Integer; nChannelId: Integer; nErrorCode: Integer; sErrorMsg: String); override;
  procedure OnVaxErrorLog(sFuncName: String; nErrorCode: Integer; sErrorMsg: String); override;

  private
  procedure OnLineRegisterTryingREC(sLineName: String);
  procedure OnLineRegisterFailedREC(sLineName: String; nStatusCode: Integer; sReasonPhrase: String);
  procedure OnLineRegisterSuccessREC(sLineName: String);

  private
  procedure OnInitialize();
  procedure OnStatusLog(sMsg: string);

  public
  procedure GenStatusLog(sStatus: string);

end;

implementation

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  Uses VaxCallSession, VaxUserList, VaxCallSessionList, VaxUser,
       VaxLine, VaxCallParking, VaxSettingDefault,
       VaxDefine, VaxRoom, VaxStealthListen;

  var m_objSessionList: TVaxCallSessionList = nil;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  constructor TVaxServerSIP.Create();
  begin
    inherited Create();
    mVaxSIP := self;

    m_bInitialized := False;
    ResetAll();
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TVaxServerSIP.StartServer(nListenPort: Integer): boolean;
  begin

    StopServer();

    Inherited SetLicenseKey('TRIAL-LICENSE-KEY');

    if(Inherited Initialize('') = False) Then
        Exit(False);

    if(Inherited OpenNetworkUDP('', nListenPort) = False) Then
    begin
        UnInitialize();
        Exit(False);
    end;

    if(Inherited OpenNetworkTCP('', nListenPort) = False) Then
    begin
        UnInitialize();
        Exit(False);
    end;

    if(Inherited OpenNetworkTLS('', nListenPort + 1, '') = False) Then
    begin
        UnInitialize();
        Exit(False);
    end;

    // Inherited AttackDetectScanSIP('demo.vaxvoip.com');
    // Inherited AttackDetectFloodSIP(1000);
    // Inherited AttackDetectBruteForceSIP(10, 2);

    Inherited SetListenPortRangeRTP(20000, 30000);
    Inherited AudioSessionLost(true, 30); // Enable voice session lost for 30 seconds.
    Inherited VideoCOMM(true);

    Inherited AddLine('AnonymousUDP', TVaxServerCOM.VAX_LINE_TYPE_UDP, '', '', '', '', '', '255.255.255.255', -1, '');
    Inherited AddLine('AnonymousTCP', TVaxServerCOM.VAX_LINE_TYPE_UDP, '', '', '', '', '', '255.255.255.255', -1, '');
    Inherited AddLine('AnonymousTLS', TVaxServerCOM.VAX_LINE_TYPE_UDP, '', '', '', '', '', '255.255.255.255', -1, '');

    Inherited AddCallPickUpGroupRouteUID('#5#'); // OnIncomingCall(), OnCallSessionTransferBlind(), sRouteUID Parameter.
    Inherited AddStealthListenRouteUID('#1#');   // OnIncomingCall(), OnCallSessionTransferBlind(), sRouteUID Parameter.
    Inherited AddCallParkRouteUID('#3#');        // OnIncomingCall(), OnCallSessionTransferBlind(), sRouteUID Parameter.

    //Inherited ActivatePushSIP(true, 1);

    m_bInitialized := True;

    OnInitialize();
    Result := True;

  end;

  ////////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.StopServer();
  begin

    m_bInitialized := false;

    Inherited UnInitialize();
    ResetAll();

  end;

  function TVaxServerSIP.IsInitialized(): Boolean;
  begin
    Result :=  m_bInitialized;
  end;

  ////////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////////////////////////////////////////////////////////

  function TVaxServerSIP.AcceptCallSession(nSessionId: Integer; sCallerName: String; sCallerId: String; sDialNo: String; sToPeerName: String; sRouteUID: String): Boolean;
  begin

    Result := inherited AcceptCallSession(nSessionId, sCallerName, sCallerId, sDialNo, sToPeerName, sRouteUID, 30);
    if Result = False then Exit(False);

    if IsActivatedREC() then
    begin
      Result := inherited ConnectToServerREC(nSessionId, 0, sCallerName, sCallerId, sDialNo, LINE_NAME_REC, 30);
      if Result = False then Exit(False);
    end;

    Result := True;

  end;

  ////////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////////////////////////////////////////////////////////

  function TVaxServerSIP.IsLineREC(sLineName: String): Boolean;
  begin
    Result := (sLineName = LINE_NAME_REC);
  end;

  procedure  TVaxServerSIP.DeactivateREC;
  begin
    inherited UnRegisterLine(LINE_NAME_REC);
    m_bIsActivatedREC := False;
  end;

  /////////////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////////////

  function TVaxServerSIP.ActivateREC(sAuthLogin: String; sAuthPwd: String; sServerAddr: String; nServerPort: Integer; bRegister: Boolean): Boolean;
  begin

    inherited RemoveLine(LINE_NAME_REC);

    Result := inherited AddLine(LINE_NAME_REC, TVaxServerCOM.VAX_LINE_TYPE_UDP, sAuthLogin, sAuthLogin, sAuthLogin, sAuthLogin, '', sServerAddr, nServerPort, '32041');
    if(Result = false) then Exit;

    if(bRegister = true) then
      Result := inherited RegisterLine(LINE_NAME_REC, 1800);

    m_bIsActivatedREC := True;

  end;

  /////////////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////////////

  function TVaxServerSIP.IsActivatedREC: Boolean;
  begin
    Result := m_bIsActivatedREC;
  end;

  ///////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////

  function TVaxServerSIP.DialToneToCallSession(bEnable: Boolean; nSessionId: Integer) : Boolean;
  begin

    if(m_nDialToneId = -1) then
      Exit(True);

    Result := inherited DialToneToCallSession(bEnable, nSessionId, m_nDialToneId);

  end;

  ///////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////

  function TVaxServerSIP.LoadMusicDialTone(sFileName : string) : Boolean;
  begin

    if(m_nDialToneId <> -1) Then
    begin
      UnLoadWaveID(m_nDialToneId);
      m_nDialToneId := -1;
    end;

    m_nDialToneId := inherited LoadWaveFile(sFileName);

    if(m_nDialToneId = -1) Then
      Exit(False);

    Result := True;

  end;

  procedure TVaxServerSIP.LoadMusicFiles(sMusicHold: String; sMusicDialTone: String; sMusicCallPark: String);
  begin
    Inherited LoadMusicHold(sMusicHold);
    LoadMusicDialTone(sMusicDialTone);
    TVaxCallParking.LoadMusic(sMusicCallPark);
    TVaxStealthListen.LoadMusic(sMusicCallPark);
  end;

  ///////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.ResetAll();
  begin

    m_bIsActivatedREC := False;
    m_nDialToneId := -1;

    if(m_objSessionList <> nil) Then
    Begin
      m_objSessionList.Destroy();
      m_objSessionList := nil;
    End;

    m_objSessionList := TVaxCallSessionList.Create();

    TVaxUser.ResetAll();
    TVaxCallSession.ResetAll();

    TVaxStealthListen.ResetAll();
    TVaxCallParking.ResetAll();

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnCallSessionCreated(nSessionId: Integer; nReasonCode: Integer);
  var
    objCallSession: TVaxCallSession;
  begin

    m_objSessionList.AddSession(nSessionId);

    objCallSession := m_objSessionList.GetSession(nSessionId);
    if(objCallSession <> nil) Then objCallSession.OnCallSessionCreated(nSessionId, nReasonCode);

  end;


  procedure TVaxServerSIP.OnCallSessionClosed(nSessionId: Integer; nChannelId: Integer; nReasonCode: Integer);
  var
    objCallSession: TVaxCallSession;
  begin

    objCallSession := m_objSessionList.GetSession(nSessionId);
    if(objCallSession <> nil) Then objCallSession.OnCallSessionClosed(nSessionId, nChannelId, nReasonCode);

    m_objSessionList.RemoveSession(nSessionId);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnCallSessionAccessAudioPCM(nSessionId: Integer; nChannelId: Integer; nAccessType: Integer; pDataPCM: PByte; nSizePCM: Integer; nTypePCM: Integer);
  var
    objCallSession: TVaxCallSession;
  begin

    objCallSession := m_objSessionList.GetSession(nSessionId);
    if(objCallSession = nil) Then Exit();

    objCallSession.OnCallSessionAccessAudioPCM(nSessionId, nChannelId, nAccessType, pDataPCM, nSizePCM);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnCallSessionPushSIP(nSessionId: Integer; nChannelId: Integer; sCallerName: String; sCallerId: String; nProviderPN: Integer; sProjectPN: String; sDevicePN: String);
  begin

    // Use OnCallSessionPushSIP() event to send Push-Notification to Android and iOS based devices.
    // OnCallSessionPushSIP() event triggers only if softphone or SIP client supports Push-SIP RFC.
    // VaxVoIP Phone SDK iOS & Android already support Push-SIP RFC.

    //Inherited ActivatePushSIP(true, 1); VaxServerSDK Exported method. Please search in this Class. Please use it in StartServer() function.

    (*

    if(nProviderPN = TVaxServerCOM.VAX_PUSH_PROVIDER_IOS)  Then
    begin
      //Send Notification by using HTTP, Curl.exe or any other method to iOS device. sDevicePN is device token. sProjectPN is APP Name.
    end
    else if (nProviderPN = TVaxServerCOM.VAX_PUSH_PROVIDER_ANDROID) Then
    begin
      //Send Notification by using HTTP, Curl.exe or any other method to Android device. sDevicePN is device token. sProjectPN is APP Name.
    end;

    *)

  end;

  ///////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnIncomingCall(nSessionId: Integer; sCallerName: String; sCallerId: String; sDialNo: String; nFromPeerType: Integer; sFromPeerName: String; sRouteUID: String; nRouteType: Integer; sRouteName: String; sUserAgentName: String; sFromIP: String; nFromPort: Integer);
  var
    objCallSession: TVaxCallSession;
  begin

    ////////////////////////////////////////////////////////////////////////////

    if(Length(sRouteUID) <> 0) Then
    begin

      if(nRouteType = TVaxServerCOM.VAX_ROUTE_TYPE_CALL_PARK) Then
      begin
        TVaxCallParking.OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, sRouteUID);
        Exit();
      end;

      if(nRouteType = TVaxServerCOM.VAX_ROUTE_TYPE_STEALTH_LISTEN) Then
      begin
        TVaxStealthListen.OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, nFromPeerType, sFromPeerName, sRouteUID);
        Exit();
      end;

      AcceptCallSession(nSessionId, sCallerName, sCallerId, sDialNo, '', sRouteUID);
      Exit();

    end;

    ////////////////////////////////////////////////////////////////////////////

    objCallSession := m_objSessionList.GetSession(nSessionId);
    if(objCallSession = nil) Then Exit();

    DialToneToCallSession(True, nSessionId);
    objCallSession.OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, nFromPeerType, sFromPeerName, sUserAgentName, sFromIP, nFromPort);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnCallSessionTransferBlind(nTransfererSessionId: Integer; nTransfererChannelId: Integer; sTransferer: String; sTransferee: String; sTransferTo: String; sRouteUID: String; nRouteType: Integer; sRouteName: String);
  var
    objCallSession: TVaxCallSession;
  begin

    ////////////////////////////////////////////////////////////////////////////

    if(Length(sRouteUID) <> 0) Then
    begin

      if(nRouteType = TVaxServerCOM.VAX_ROUTE_TYPE_CALL_PARK) Then
      begin
        TVaxCallParking.OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferee, sTransferTo, sRouteUID);
        Exit();
      end;

      Inherited AcceptTransferBlind(nTransfererSessionId, sTransferee, sTransferee, sTransferTo, '', sRouteUID, 30);
      Exit();

    end;

    ////////////////////////////////////////////////////////////////////////////

    objCallSession := m_objSessionList.GetSession(nTransfererSessionId);
    if(objCallSession = nil) Then Exit();

    objCallSession.OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferer, sTransferee, sTransferTo);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnCallSessionTransferConsult(nTransfererSessionId: Integer; nTransfererChannelId: Integer; nTransferToSessionId: Integer; nTransferToChannelId: Integer; sTransferer: String; sTransferee: String; sTransferTo: String);
  var
    objCallSession: TVaxCallSession;
  begin
    objCallSession := m_objSessionList.GetSession(nTransfererSessionId);
    if(objCallSession = nil) Then Exit();

    objCallSession.OnCallSessionTransferConsult(nTransfererSessionId, nTransfererChannelId, nTransferToSessionId, nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnRegisterUser(sUserLogin: String; sDomain: String; sUserAgentName: String; sFromIP: String; nFromPort: Integer; nRegId: Integer);
  begin
    TVaxUser.OnRegisterUser(sUserLogin, sDomain, sUserAgentName, sFromIP, nFromPort, nRegId);
  end;

  procedure TVaxServerSIP.OnRegisterUserSuccess(sUserLogin: String; sFromIP: String; nFromPort: Integer; nRegId: Integer);
  begin
    TVaxUser.OnRegisterUserSuccess(sUserLogin, sFromIP, nFromPort, nRegId);
  end;

  procedure TVaxServerSIP.OnRegisterUserFailed(sUserLogin: String; sFromIP: String; nFromPort: Integer; nRegId: Integer);
  begin
    TVaxUser.OnRegisterUserFailed(sUserLogin, sFromIP, nFromPort, nRegId);
  end;

  procedure TVaxServerSIP.OnUnRegisterUser(sUserLogin: String);
  begin
    TVaxUser.OnUnRegisterUser(sUserLogin);
  end;

  ///////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnLineRegisterTrying(sLineName: String);
  begin

    if(IsLineREC(sLineName)) then
    begin
      OnLineRegisterTryingREC(sLineName);
      Exit
    end;

    TVaxLine.OnLineRegisterTrying(sLineName);

  end;

  procedure TVaxServerSIP.OnLineRegisterFailed(sLineName: String; nStatusCode: Integer; sReasonPhrase: String);
  begin

    if(IsLineREC(sLineName)) then
    begin
      OnLineRegisterFailedREC(sLineName, nStatusCode, sReasonPhrase);
      Exit
    end;

    TVaxLine.OnLineRegisterFailed(sLineName, nStatusCode, sReasonPhrase);

  end;

  procedure TVaxServerSIP.OnLineRegisterSuccess(sLineName: String);
  begin

    if(IsLineREC(sLineName)) then
    begin
      OnLineRegisterSuccessREC(sLineName);
      Exit
    end;

    TVaxLine.OnLineRegisterSuccess(sLineName);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnDetectedDigitDTMF(nSessionId: Integer; nChannelId: Integer; sDigitDTMF: String; nTypeDTMF: Integer);
  var
    objCallSession: TVaxCallSession;
  begin
    objCallSession := m_objSessionList.GetSession(nSessionId);
    if(objCallSession = nil) Then Exit();

    objCallSession.OnDetectedDigitDTMF(nSessionId, nChannelId, sDigitDTMF);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnCallSessionDetectedAMD(nSessionId: Integer; nChannelId: Integer; bIsHuman: Boolean);
  var
    objCallSession: TVaxCallSession;
  begin
    objCallSession := m_objSessionList.GetSession(nSessionId);
    if(objCallSession = nil) Then Exit();

    objCallSession.OnCallSessionDetectedAMD(nSessionId, nChannelId, bIsHuman);
  end;

  procedure TVaxServerSIP.OnStealthListenerAttached(nAttachType: Integer; nStealthSessionId: Integer; nAttachSessionId: Integer; nAttachChannelId: Integer);
  begin
    TVaxStealthListen.OnStealthListenerAttached(nAttachType, nStealthSessionId, nAttachSessionId, nAttachChannelId);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnChatMessageText(nChatMsgId: Integer; sMsgFrom: String; sMsgTo: String; sMsgText: String; nFromPeerType: Integer; sFromPeerName: String; sFromIP: String; nFromPort: Integer);
  begin
    Inherited AcceptChatMessage(nChatMsgId, sMsgTo);
  end;

  procedure TVaxServerSIP.OnChatMessageTyping(nChatMsgId: Integer; sMsgFrom: String; sMsgTo: String; bIsTypingStart: Boolean; nFromPeerType: Integer; sFromPeerName: String; sFromIP: String; nFromPort: Integer);
  begin
    Inherited AcceptChatMessage(nChatMsgId, sMsgTo);
  end;

  procedure TVaxServerSIP.OnChatStatusSubscribe(nSubscrbId: Integer; sMsgFrom: String; sMsgTo: String; nFromPeerType: Integer; sFromPeerName: String; sFromIP: String; nFromPort: Integer);
  begin
    Inherited AcceptChatStatusSubscribe(nSubscrbId, sMsgFrom, sMsgTo, sMsgTo);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnBusyLampSubscribe(nSubscrbId: Integer; sUserName: String; sToUserName: String; sFromIP: String; nFromPort: Integer);
  begin
    Inherited BusyLampSubscribeAccept(nSubscrbId, True, 30)
  end;

  procedure TVaxServerSIP.OnBusyLampSendStatus(sFromUserName: String; sToUserName: String; nStateId: Integer);
  begin
    Inherited BusyLampSendStatus(sFromUserName, sToUserName, nStateId);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnLineRegisterTryingREC(sLineName: String);
  begin
    m_objRecDlgSIP.OnLineRegisterTryingREC(sLineName);
  end;

  procedure TVaxServerSIP.OnLineRegisterFailedREC(sLineName: String; nStatusCode: Integer; sReasonPhrase: String);
  begin
    m_objRecDlgSIP.OnLineRegisterFailedREC(sLineName, nStatusCode, sReasonPhrase);
  end;

  procedure TVaxServerSIP.OnLineRegisterSuccessREC(sLineName: String);
  begin
    m_objRecDlgSIP.OnLineRegisterSuccessREC(sLineName);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnVaxErrorLog(sFuncName: String; nErrorCode: Integer; sErrorMsg: String);
  var
   sMsg: String;
  begin
    if(Length(sFuncName) = 0) Then
    Begin
       OnStatusLog(sErrorMsg);
       Exit();
    End;

    sMsg := '[Method: ' + sFuncName + '] [Error Msg: ' + sErrorMsg + '] [Code: ' + IntToStr(nErrorCode) + ']';
    OnStatusLog(sMsg);
  end;

  procedure TVaxServerSIP.OnCallSessionErrorLog(nSessionId: Integer; nChannelId: Integer; nErrorCode: Integer; sErrorMsg: String);
  var
   sMsg: String;
  begin
    sMsg := '[Session: ' + IntToStr(nSessionId) + '] [Error Msg: ' + sErrorMsg + '] [Code: ' + IntToStr(nErrorCode) + ']';
    OnStatusLog(sMsg);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxServerSIP.OnInitialize();
  begin

    if (TVaxSettingDefault.LoadUserDefault() = False) then
      TVaxUser.LoadUser();

    if (TVaxSettingDefault.LoadLineDefault() = False) then
      TVaxLine.LoadLine();

    if (TVaxSettingDefault.LoadRoomDefault() = False) then
      TVaxRoom.LoadRoom();

  end;

  procedure TVaxServerSIP.GenStatusLog(sStatus: string);
  begin
    OnStatusLog(sStatus);
  end;

  procedure TVaxServerSIP.OnStatusLog(sMsg: string);
  begin
    m_objStatusLogDlg.OnStatusLog(sMsg);
  end;

end.

