unit VaxUser;

interface

uses
  System.Classes,
  StoreUser,

  VaxUserList, VaxCallSession,
  VaxDefine, VaxHashText;

type TVaxUser = class(TVaxHashTextData)

  private
  m_bBlocked, m_bRegistered: Boolean;
  m_nVoiceTypeId: Integer;

  public
  class procedure OnRegisterUser(sUserLogin: String; sDomain: String; sUserAgentName: String; sFromIP: String; nFromPort: Integer; nRegId: Integer); static;
  class procedure OnRegisterUserSuccess(sUserLogin: String; sFromIP: String; nFromPort: Integer; nRegId: Integer); static;
  class procedure OnRegisterUserFailed(sUserLogin: String; sFromIP: String; nFromPort: Integer; nRegId: Integer); static;
  class procedure OnUnRegisterUser(sUserLogin: string); static;

  private
  procedure OnRegister(sUserLogin: String; nRegId: Integer);
  procedure OnRegisterSuccess(sUserLogin: string);
  procedure OnRegisterFailed(sUserLogin: string);
  procedure OnUnRegister(sUserLogin: string);

  public
  constructor Create(sUserName: string; bBlocked: boolean; nVoiceTypeId: Integer);
  class procedure ResetAll();
  class function GetRegisteredUserCount(): Integer;

  private
  class function GetUser(sUserName: String): TVaxUser; static;

  public
  class function AddUser(sUserName: String; sPassword: String; sAudioCodecList: String; bBlocked: Boolean; nVoiceTypeId: Integer; sUserPrefix, sDialPrefix: String; sToPeerName: String): Boolean; static;
  class procedure RemoveUser(sUserName: String); static;
  class procedure LoadUser(); static;

  public
  class function IsUserTypeAdmin(sUserName: String): Boolean; static;
  class function GetVoiceTypeId(sUserName: String): Integer; static;

end;

implementation

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  Uses VaxServerSIP, VaxServerCOM;
  var
  m_objUserList: TVaxUserList = nil;
  m_nRegisteredUserCount: Integer = 0;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  constructor TVaxUser.Create(sUserName: string; bBlocked: boolean; nVoiceTypeId: Integer);
  begin

    if(m_objUserList = nil) then
      m_objUserList := TVaxUserList.Create();

     m_bBlocked := bBlocked;
     m_nVoiceTypeId := nVoiceTypeId;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class procedure TVaxUser.ResetAll();
  begin

    m_nRegisteredUserCount := 0;

    if(m_objUserList <> nil) Then
    begin
      m_objUserList.Destroy;
      m_objUserList := nil;
    end;

    m_objUserList :=  TVaxUserList.Create();

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class function TVaxUser.GetUser(sUserName: String): TVaxUser;
  begin
    Result := TVaxUser(m_objUserList.GetUser(sUserName));
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class procedure TVaxUser.OnRegisterUser(sUserLogin: String; sDomain: String; sUserAgentName: String; sFromIP: String; nFromPort: Integer; nRegId: Integer);
  var
    objUser: TVaxUser;
  begin
    objUser := TVaxUser.GetUser(sUserLogin);

    if(objUser = nil) Then
    begin
      TVaxServerSIP.mVaxSIP.RejectRegister(nRegId, 404, TVaxDefine.NOT_FOUND);
      TVaxServerSIP.mVaxSIP.GenStatusLog(sUserLogin + ' ' + TVaxDefine.USER_NOT_FOUND);
      Exit();
    end;

    objUser.OnRegister(sUserLogin, nRegId);
  end;

  procedure TVaxUser.OnRegister(sUserLogin: String; nRegId: Integer);
  begin

    If m_bBlocked = True then
    begin
      TVaxServerSIP.mVaxSIP.RejectRegister(nRegId, 404, TVaxDefine.NOT_FOUND);
      Exit;
    end;

    TVaxServerSIP.mVaxSIP.AuthRegister(nRegId);
    TVaxServerSIP.mVaxSIP.GenStatusLog(sUserLogin + ' ' + TVaxDefine.USER_REG_TRYING);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class procedure TVaxUser.OnRegisterUserSuccess(sUserLogin: String; sFromIP: String; nFromPort: Integer; nRegId: Integer);
  var
    objUser: TVaxUser;
  begin
    objUser := TVaxUser.GetUser(sUserLogin);
    if(objUser = nil) Then Exit();

    objUser.OnRegisterSuccess(sUserLogin);
  end;

  procedure TVaxUser.OnRegisterSuccess(sUserLogin: string);
  begin

    if (m_bRegistered = false) then
      m_nRegisteredUserCount := m_nRegisteredUserCount + 1;

    m_bRegistered := True;
    TVaxServerSIP.mVaxSIP.GenStatusLog(sUserLogin + ' ' + TVaxDefine.USER_REG_SUCCESS);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class procedure TVaxUser.OnRegisterUserFailed(sUserLogin: String; sFromIP: String; nFromPort: Integer; nRegId: Integer);
  var
    objUser: TVaxUser;
  begin

    objUser := TVaxUser.GetUser(sUserLogin);
    if(objUser = nil) Then Exit();

    objUser.OnRegisterFailed(sUserLogin);

  end;

  procedure TVaxUser.OnRegisterFailed(sUserLogin: string);
  begin
    TVaxServerSIP.mVaxSIP.GenStatusLog(sUserLogin + ' ' + TVaxDefine.USER_REG_FAILED);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class procedure TVaxUser.OnUnRegisterUser(sUserLogin: String);
  var
   objUser: TVaxUser;
  begin
    objUser := TVaxUser.GetUser(sUserLogin);
    if(objUser = nil) Then Exit();

    objUser.OnUnRegister(sUserLogin);
  end;

  procedure TVaxUser.OnUnRegister(sUserLogin: string);
  begin

    if (m_bRegistered = True) then
      m_nRegisteredUserCount := m_nRegisteredUserCount - 1;

    m_bRegistered := False;
    TVaxServerSIP.mVaxSIP.GenStatusLog(sUserLogin + ' ' + TVaxDefine.USER_UNREGISTER);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  class function TVaxUser.AddUser(sUserName: String; sPassword: String; sAudioCodecList: String; bBlocked: Boolean; nVoiceTypeId: Integer; sUserPrefix, sDialPrefix: String; sToPeerName: String): Boolean;
  begin

    if((TVaxServerSIP.mVaxSIP.IsInitialized() = False) or (Length(sUserName) = 0)) Then
      Exit(False);

    RemoveUser(sUserName);

    if(TVaxServerSIP.mVaxSIP.AddUser(sUserName, sPassword, sAudioCodecList) = False) Then
      Exit(False);

    TVaxCallSession.AddUserRoute(sUserName, sUserPrefix, sDialPrefix, sToPeerName);
    m_objUserList.AddUser(sUserName, bBlocked, nVoiceTypeId);

    TStoreUser.AddUser(sUserName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
    Result := True;

  end;

  class procedure TVaxUser.RemoveUser(sUserName: String);
  begin

    if((TVaxServerSIP.mVaxSIP.IsInitialized() = False) or (Length(sUserName) = 0)) Then
      Exit();

    TVaxServerSIP.mVaxSIP.RemoveUser(sUserName);

    TVaxCallSession.RemoveUserRoute(sUserName);
    m_objUserList.RemoveUser(sUserName);

    TStoreUser.RemoveUser(sUserName);

  end;

  class procedure TVaxUser.LoadUser();
  var
    sUserName, sPassword, sUserPrefix: string ;
    sDialPrefix, sAudioCodecList, sToPeerName: string;

    nVoiceTypeId: Integer;
    bBlocked: Boolean;

    objListUsers: TStringList;
  begin

    if(TVaxServerSIP.mVaxSIP.IsInitialized() = False) Then
      Exit();

    objListUsers :=  TStringList.Create();
    TStoreUser.GetUserAll(objListUsers);

    if(objListUsers.Count = 0) Then
      Exit();

    sPassword := ''; sAudioCodecList := '';
    sUserPrefix := ''; sDialPrefix := ''; sToPeerName := '';

    bBlocked := false; nVoiceTypeId := TVaxServerCOM.VAX_AUDIO_TYPE_NORMAL;

    for sUserName in objListUsers do
    begin
      TStoreUser.GetUser(sUserName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
      AddUser(sUserName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
    end;

  end;

  ///////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////

  class function TVaxUser.IsUserTypeAdmin(sUserName: String): Boolean;
  var
    objUser: TVaxUser;
  begin
    objUser := GetUser(sUserName);
    if(objUser = nil) Then Exit(False);
    Result := (objUser.m_nVoiceTypeId = TVaxServerCOM.VAX_AUDIO_TYPE_ADMIN);
  end;

  class function TVaxUser.GetVoiceTypeId(sUserName: String): Integer;
  var
    objUser: TVaxUser;
  begin
    objUser := GetUser(sUserName);
    if(objUser = nil) Then Exit(TVaxServerCOM.VAX_AUDIO_TYPE_NORMAL);
    Result := objUser.m_nVoiceTypeId;
  end;

  ///////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////

  class function TVaxUser.GetRegisteredUserCount(): Integer;
  begin
    Result := m_nRegisteredUserCount;
  end;

end.
