﻿unit StoreUser;

Interface

uses

  Winapi.Windows, Winapi.Messages,
  System.SysUtils, System.Classes, StoreRegistry,
  VaxServerCOM;

type TStoreUser = class

  private
  const USER_ROOT_PATH = 'Users\\';

  public
  class function AddUser(sUserName: String; sPassword: String; sAudioCodecList: String; bBlocked: Boolean; nVoiceTypeId: Integer; sUserPrefix, sDialPrefix: String; sToPeerName: String): Boolean; static;
  class procedure GetUser(sUserName: String; var sPassword: String; var sAudioCodecList: String; var bBlocked: Boolean; var nVoiceTypeId: Integer; var sUserPrefix: String; var sDialPrefix: String; var sToPeerName: String); static;

  class procedure RemoveUser(sUserName: String); static;
  class procedure GetUserAll(var objList: TStringList); static;

  public
  class function IsDataStored(): Boolean; static;

  private
  class function AddUserRoute(sUserName: String; sUserPrefix: String; sDialPrefix: String; sToPeerName: String): Boolean; static;
  class function RemoveUserRoute(sUserName: String): Boolean; static;
  class procedure GetUserRoute(sUserName: String; var sUserPrefix: String; var sDialPrefix: String; var sToPeerName: String); static;

end;


implementation

  //////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////

  class function TStoreUser.AddUser(sUserName: String; sPassword: String; sAudioCodecList: String; bBlocked: Boolean; nVoiceTypeId: Integer; sUserPrefix, sDialPrefix: String; sToPeerName: String): Boolean;
  var
    sKeyPath: String;
    objStore: TStoreRegistry;
  begin

    if Length(sUserName) = 0 then
    begin
      Exit(False);
    end;

    sKeyPath := USER_ROOT_PATH + sUserName;
    objStore := TStoreRegistry.Create();

    objStore.SetTEXT(sKeyPath, 'UserName', sUserName);
    objStore.SetTEXT(sKeyPath, 'Password', sPassword);

    objStore.SetTEXT(sKeyPath, 'AudioCodecList', sAudioCodecList);

    objStore.SetBOOL(sKeyPath, 'Blocked', bBlocked);
    objStore.SetINT(sKeyPath, 'VoiceTypeId', nVoiceTypeId);
    objStore.Destroy();

    TStoreUser.AddUserRoute(sUserName, sUserPrefix, sDialPrefix, sToPeerName);
    Result := True;

  end;

  class procedure TStoreUser.GetUser(sUserName: String; var sPassword: String; var sAudioCodecList: String; var bBlocked: Boolean; var nVoiceTypeId: Integer; var sUserPrefix: String; var sDialPrefix: String; var sToPeerName: String);
  var
    sKeyPath: String;
    objStore: TStoreRegistry;
  begin

    sKeyPath := USER_ROOT_PATH + sUserName;
    objStore := TStoreRegistry.Create();

    objStore.GetTEXT(sKeyPath, 'Password', sPassword, '');
    objStore.GetTEXT(sKeyPath, 'AudioCodecList', sAudioCodecList, '');

    objStore.GetBOOL(sKeyPath, 'Blocked', bBlocked, False);
    objStore.GetINT(sKeyPath, 'VoiceTypeId', nVoiceTypeId, TVaxServerCOM.VAX_AUDIO_TYPE_NORMAL);
    objStore.Destroy();

    TStoreUser.GetUserRoute(sUserName, sUserPrefix, sDialPrefix, sToPeerName);

  end;

  class procedure TStoreUser.RemoveUser(sUserName: String);
  var
    sKeyPath: String;
    objStore: TStoreRegistry;
  begin

    if Length(sUserName) = 0 then
    begin
      Exit();
    end;

    sKeyPath := USER_ROOT_PATH + sUserName;
    objStore := TStoreRegistry.Create();

    objStore.DeleteSubKey(sKeyPath);
    objStore.Destroy();

    TStoreUser.RemoveUserRoute(sUserName);

  end;

  class procedure TStoreUser.GetUserAll(var objList: TStringList);
  var
    sKeyPath: String;
    objStore: TStoreRegistry;
  begin

    sKeyPath := USER_ROOT_PATH;
    objStore := TStoreRegistry.Create();

    objStore.EnumSubKeys(sKeyPath, objList);
    objStore.Destroy();

  end;

  //////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////

  class function TStoreUser.AddUserRoute(sUserName: String; sUserPrefix: String; sDialPrefix: String; sToPeerName: String): Boolean;
  var
    sKeyPath: String;
    objStore: TStoreRegistry;
  begin

    if Length(sUserName) = 0 then
    begin
      Exit(False);
    end;

    sKeyPath := USER_ROOT_PATH + sUserName;
    objStore := TStoreRegistry.Create();

    objStore.SetTEXT(sKeyPath, 'UserPrefix', sUserPrefix);
    objStore.SetTEXT(sKeyPath, 'DialPrefix', sDialPrefix);
    objStore.SetTEXT(sKeyPath, 'ToPeerName', sToPeerName);

    objStore.Destroy();
    Result := True;

  end;

  class procedure TStoreUser.GetUserRoute(sUserName: String; var sUserPrefix: String; var sDialPrefix: String; var sToPeerName: String);
  var
    sKeyPath: String;
    objStore: TStoreRegistry;
  begin

    sKeyPath := USER_ROOT_PATH + sUserName;
    objStore := TStoreRegistry.Create();

    objStore.GetTEXT(sKeyPath, 'UserPrefix', sUserPrefix, '');
    objStore.GetTEXT(sKeyPath, 'DialPrefix', sDialPrefix, '');
    objStore.GetTEXT(sKeyPath, 'ToPeerName', sToPeerName, '');

    objStore.Destroy();

  end;

  class function TStoreUser.RemoveUserRoute(sUserName: String): Boolean;
  var
    sKeyPath: String;
    objStore: TStoreRegistry;
  begin

    if Length(sUserName) = 0 then
    begin
      Exit(False);
    end;

    sKeyPath := USER_ROOT_PATH + sUserName;
    objStore := TStoreRegistry.Create();

    objStore.DeleteValue(sKeyPath, 'UserPrefix');
    objStore.DeleteValue(sKeyPath, 'DialPrefix');
    objStore.DeleteValue(sKeyPath, 'ToPeerName');

    objStore.Destroy();

    Result := True;

  end;

  //////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////

  class function TStoreUser.IsDataStored(): Boolean;
  var
    objStore: TStoreRegistry;
  begin
    objStore := TStoreRegistry.Create();
    Result := objStore.IsKeyExist(USER_ROOT_PATH);
    objStore.Destroy();
  end;

end.
